//
//  ZGHUDHelper.m
//  ZIMExampleLegacy (iOS)
//
//  Created by Patrick Fu on 2021/6/26.
//  Copyright © 2021 Zego. All rights reserved.
//

#import "ZGHUDHelper.h"
#import "MBProgressHUD.h"

@implementation ZGHUDHelper

+ (void)showMessage:(NSString *)message{
    [self showMessage:message done:nil];
}

+ (void)showMessage:(NSString *)message done:(void(^)(void))doneHandler{
    dispatch_async(dispatch_get_main_queue(), ^{
        MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[UIApplication sharedApplication].keyWindow animated:YES];
        
        hud.mode = MBProgressHUDModeText;
        hud.detailsLabel.text = message;
        hud.detailsLabel.font = [UIFont systemFontOfSize:15];
        [hud hideAnimated:YES afterDelay:1.5f];
        hud.completionBlock = doneHandler;
    });
}

+ (void)showIndeterminateMessage:(NSString *)message done:(void (^)(void))doneHandler {
    dispatch_async(dispatch_get_main_queue(), ^{
        MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[UIApplication sharedApplication].keyWindow animated:YES];
        hud.mode = MBProgressHUDModeIndeterminate;
        hud.detailsLabel.text = message;
        [hud hideAnimated:YES afterDelay:1.5f];
        hud.completionBlock = doneHandler;
    });
}

+ (void)showCustomMessage:(NSString *)message customView:(UIView *)customView done:(void(^)(void))doneHandler{
    dispatch_async(dispatch_get_main_queue(), ^{
        MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[UIApplication sharedApplication].keyWindow animated:YES];
        hud.mode = MBProgressHUDModeCustomView;
        hud.customView = customView;
        hud.label.text = message;
        [hud hideAnimated:YES afterDelay:1.5];
        hud.completionBlock = doneHandler;
    });
}

+ (void)showNetworkLoading {
    dispatch_async(dispatch_get_main_queue(), ^{
        [MBProgressHUD showHUDAddedTo:[UIApplication sharedApplication].keyWindow animated:YES];
    });
}

+ (void)hideNetworkLoading {
    dispatch_async(dispatch_get_main_queue(), ^{
        for (UIView *subView in [UIApplication sharedApplication].keyWindow.subviews) {
            if ([subView isKindOfClass:[MBProgressHUD class]]) {
                MBProgressHUD *hud = (MBProgressHUD *)subView;
                if (hud.mode == MBProgressHUDModeIndeterminate && hud.label.text == nil) {
                    hud.removeFromSuperViewOnHide = YES;
                    [hud hideAnimated:YES];
                }
            }
        }
    });
}

@end
