//
//  NotificationService.m
//  ExampleExtension
//
//  Created by 武耀琳 on 2023/7/20.
//

#import "NotificationService.h"
#import <Intents/Intents.h>
#import "NSDictionary+MyLog.h"
#import <UIKit/UIKit.h>



@interface NotificationService ()

@property (nonatomic, strong) void (^contentHandler)(UNNotificationContent *contentToDeliver);
@property (nonatomic, strong) UNMutableNotificationContent *bestAttemptContent;

@end

@implementation NotificationService

- (void)didReceiveNotificationRequest:(UNNotificationRequest *)request withContentHandler:(void (^)(UNNotificationContent * _Nonnull))contentHandler {
    self.contentHandler = contentHandler;
    self.bestAttemptContent = [request.content mutableCopy];
    // 标题
    NSString *title = self.bestAttemptContent.title;
    // 副标题
    NSString *subtitle = self.bestAttemptContent.subtitle;
    // 内容
    NSString *body = self.bestAttemptContent.body;

    // 取出发送推送消息附带的 payload 字符串
    NSString *payload = [self.bestAttemptContent.userInfo objectForKey:@"payload"];

    if(payload == nil){
        self.contentHandler(self.bestAttemptContent);
        return;
    }
    
    
    // 解析 json 字符串,并转为 NSDictionary
    NSData *jsonData = [payload dataUsingEncoding:NSUTF8StringEncoding];
    NSError *error = nil;
    NSDictionary *payload_json_map = [NSJSONSerialization JSONObjectWithData:jsonData options:kNilOptions error:&error];
    if (error) {
        self.contentHandler(self.bestAttemptContent);
        return;
    }
    

//   NSString *customId =  [self.bestAttemptContent.userInfo objectForKey:@"customId"];
//
//    if(customId != nil){
//        [UNUserNotificationCenter.currentNotificationCenter getDeliveredNotificationsWithCompletionHandler:^(NSArray<UNNotification *> * _Nonnull notifications) {
//            NSString *target_id = nil;
//            for (int index = 0; index < notifications.count; index ++) {
//                if([[notifications[index].request.content.userInfo objectForKey:@"customId"] isEqual: customId]){
//                    target_id = notifications[index].request.identifier;
//                    break;
//                }
//            }
//            if(target_id != nil){
//                [UNUserNotificationCenter.currentNotificationCenter removeDeliveredNotificationsWithIdentifiers:@[target_id]];
//            }
//            contentHandler(self.bestAttemptContent);
//        }];
//    }else{
//        contentHandler(self.bestAttemptContent);
//    }

    [self downloadAvatarWithPayloadMap:payload_json_map completionHandle:^(NSData *avatarData, NSURL *avatarLocalURL) {
        [self downloadImageAttachmentWithPayloadMap:payload_json_map completionHandle:^(NSData *imageAttachmentData, NSURL *imageAttachmentLocalURL) {
            //UNMutableNotificationContent *a_messageContent = [request.content mutableCopy];
            if(imageAttachmentLocalURL){
                UNNotificationAttachment * attachment = [UNNotificationAttachment attachmentWithIdentifier:@"myAttachment" URL:imageAttachmentLocalURL options:nil error:nil];
                self.bestAttemptContent.attachments = @[attachment];
            }
            if(@available(iOS 15.0, *)){
                if(avatarData != nil){

                    // 将图片数据转换成INImage（需要 #import <Intents/Intents.h>）
                    INImage *avatar = [INImage imageWithImageData:avatarData];
                    // 创建发信对象
                    INPersonHandle *messageSenderPersonHandle = [[INPersonHandle alloc] initWithValue:@"" type:INPersonHandleTypeUnknown];
                    NSPersonNameComponents *components = [[NSPersonNameComponents alloc] init];
                    INPerson *messageSender = [[INPerson alloc] initWithPersonHandle:messageSenderPersonHandle
                                                                      nameComponents:components
                                                                         displayName:title
                                                                               image:avatar
                                                                   contactIdentifier:nil
                                                                    customIdentifier:nil
                                                                                isMe:NO
                                                                      suggestionType:INPersonSuggestionTypeNone];
                    // 创建自己对象
                    INPersonHandle *mePersonHandle = [[INPersonHandle alloc] initWithValue:@"" type:INPersonHandleTypeUnknown];
                    INPerson *mePerson = [[INPerson alloc] initWithPersonHandle:mePersonHandle
                                                                 nameComponents:nil
                                                                    displayName:nil
                                                                          image:nil
                                                              contactIdentifier:nil
                                                               customIdentifier:nil
                                                                           isMe:YES
                                                                 suggestionType:INPersonSuggestionTypeNone];


                    // 创建intent
                    INSpeakableString *speakableString = [[INSpeakableString alloc] initWithSpokenPhrase:subtitle];
                    INSendMessageIntent *intent = [[INSendMessageIntent alloc] initWithRecipients:/*@[mePerson, messageSender]*/nil
                                                                              outgoingMessageType:INOutgoingMessageTypeOutgoingMessageText
                                                                                          content:body
                                                                               speakableGroupName:speakableString
                                                                           conversationIdentifier:nil
                                                                                      serviceName:nil
                                                                                           sender:messageSender
                                                                                      attachments:nil];

                    [intent setImage:avatar forParameterNamed:@"speakableGroupName"];
                    // 创建 interaction
                    INInteraction *interaction = [[INInteraction alloc] initWithIntent:intent response:nil];
                    interaction.direction = INInteractionDirectionIncoming;
                    [interaction donateInteractionWithCompletion:nil];
                    // 创建 处理后的 UNNotificationContent
                    UNNotificationContent *newContent = [self.bestAttemptContent contentByUpdatingWithProvider:intent error:nil];
                    self.bestAttemptContent = [newContent mutableCopy];
                }

            }
            contentHandler(self.bestAttemptContent);
        }];
    }];
}

- (void)serviceExtensionTimeWillExpire {
    // Called just before the extension will be terminated by the system.
    // Use this as an opportunity to deliver your "best attempt" at modified content, otherwise the original push payload will be used.
    self.contentHandler(self.bestAttemptContent);
}



-(void)downloadAvatarWithPayloadMap:(NSDictionary *)payloadMap completionHandle:(void(^)(NSData *data,NSURL *localURL))completionHandler{
    if(@available(iOS 15.0, *)){
        NSString *avatar_url = [payloadMap objectForKey:@"avatar_url"];
        if(avatar_url == nil){
            completionHandler(nil,nil);
            return;
        }
        [self downloadWithURLString:avatar_url completionHandle:completionHandler];
    }else{
        completionHandler(nil,nil);
    }
}

-(void)downloadImageAttachmentWithPayloadMap:(NSDictionary *)payloadMap completionHandle:(void(^)(NSData *data,NSURL *localURL))completionHandler{
    NSString *imageAttachmentUrl = [payloadMap objectForKey:@"image_attachment"];
    if(imageAttachmentUrl == nil){
        completionHandler(nil,nil);
        return;
    }
    [self downloadWithURLString:imageAttachmentUrl completionHandle:completionHandler];
}


// 下载并保存图片的方法
- (void)downloadWithURLString:(NSString *)urlStr completionHandle:(void(^)(NSData *data,NSURL *localURL))completionHandler{
    __block NSData *data = nil;
    NSURL *imageURL = [NSURL URLWithString:urlStr];
    NSURLSession *session = [NSURLSession sessionWithConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration]];
    [[session downloadTaskWithURL:imageURL completionHandler:^(NSURL *temporaryFileLocation, NSURLResponse *response, NSError *error) {
        NSURL *localURL;
        if (error != nil) {
            NSLog(@"%@", error.localizedDescription);
        } else {
            NSFileManager *fileManager = [NSFileManager defaultManager];
            localURL = [NSURL fileURLWithPath:[temporaryFileLocation.path stringByAppendingString:@".png"]];
            [fileManager moveItemAtURL:temporaryFileLocation toURL:localURL error:&error];

            NSLog(@"localURL = %@", localURL);
            data = [[NSData alloc] initWithContentsOfURL:localURL];
        }
        completionHandler(data,localURL);

    }]resume];
}

- (void)redirectNSLogToDocumentFolder {
    // 如果已经连接Xcode调试则不输出到文件
    if(isatty(STDOUT_FILENO)) {
        return;
    }
    
    UIDevice *device = [UIDevice currentDevice];
    if([[device model] hasSuffix:@"Simulator"]){ //在模拟器不保存到文件中
        return;
    }
    
    // 获取Document目录下的Log文件夹，若没有则新建
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *logDirectory = [[paths objectAtIndex:0] stringByAppendingPathComponent:@"Log"];
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL fileExists = [fileManager fileExistsAtPath:logDirectory];
    if (!fileExists) {
        [fileManager createDirectoryAtPath:logDirectory  withIntermediateDirectories:YES attributes:nil error:nil];
    }
    
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"zh_CN"]];
    [formatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"]; //每次启动后都保存一个新的日志文件中
    NSString *dateStr = [formatter stringFromDate:[NSDate date]];
    NSString *logFilePath = [logDirectory stringByAppendingFormat:@"/%@.txt",dateStr];
    
    // freopen 重定向输出输出流，将log输入到文件
    freopen([logFilePath cStringUsingEncoding:NSASCIIStringEncoding], "a+", stdout);
    freopen([logFilePath cStringUsingEncoding:NSASCIIStringEncoding], "a+", stderr);
}




@end
