//
//  AppDelegate.m
//  ZIMExampleDemo
//
//  Created by Zego on 2022/1/19.
//

#import "AppDelegate.h"
#import "ZGZIMManager.h"
#import "KeyCenter.h"
#import "ConversationListDataSource.h"
#import "ZGHUDHelper.h"
#import <UserNotifications/UserNotifications.h>
#import "NSDictionary+MyLog.h"
#import "GHConsole.h"
#import <ZIM/ZIM.h>
#import "PushKit/PushKit.h"
#import "CallKitManager.h"
#import <ZPNs/ZPNs.h>
#import "CallDataManager.h"
#import <Bugly/Bugly.h>
#import "LoginViewController.h"
#import <Intents/Intents.h>
#import <CJBaseHelper/UIViewControllerCJHelper.h>
#import "GULAppEnvironmentUtil.h"

#if TARGET_OS_IOS || TARGET_OS_TV || TARGET_OS_WATCH
static NSString *const kEntitlementsAPSEnvironmentKey = @"Entitlements.aps-environment";
#else
static NSString *const kEntitlementsAPSEnvironmentKey =
    @"Entitlements.com.apple.developer.aps-environment";
#endif
static NSString *const kAPSEnvironmentDevelopmentValue = @"development";


@interface AppDelegate ()<UNUserNotificationCenterDelegate,ZPNsNotificationCenterDelegate,PKPushRegistryDelegate>

@end

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    
    // Override point for customization after application launch.
    [Bugly startWithAppId:@"7310eca12f"];
    [self redirectNSLogToDocumentFolder];
    [[GHConsole sharedConsole] startPrintLog];
    [[ZGZIMManager shared] addZIMEventDelegate:(id)[CallDataManager shared]];
    [self loadKeyCenter];
    //AppDelegate注册通知
    if (@available(iOS 10.0, *)) {
        UNUserNotificationCenter *center = [UNUserNotificationCenter currentNotificationCenter];
        center.delegate = (id)[ZPNs shared];
        [[ZPNs shared] setZPNsNotificationCenterDelegate:self];
        [center requestAuthorizationWithOptions:(UNAuthorizationOptionBadge|UNAuthorizationOptionSound|UNAuthorizationOptionAlert) completionHandler:^(BOOL granted, NSError * _Nullable error) {
            if(granted){
                dispatch_async(dispatch_get_main_queue(), ^{
                    [ZPNs version];
                    ZPNsConfig *config = [[ZPNsConfig alloc] init];
                    config.appType = 0;
                    GGLog(@"appType is 2");
                    [[ZPNs shared] setPushConfig:config];
                    [[ZPNs shared] setZPNsNotificationCenterDelegate:(id)self];
                    [[ZPNs shared] registerAPNs];
                });
                NSLog(@"用户允许发送通知");
            }
        }];
        
        ZPNsConfig *zpnsConfig = [[ZPNsConfig alloc] init];
        zpnsConfig.appType = 0;
        [[ZPNs shared] setPushConfig:zpnsConfig];
        [AppDelegate voipRegistration];
        
#if TARGET_OS_SIMULATOR && TARGET_OS_IOS
          // If APNS token is available on iOS Simulator, we must use the sandbox profile
          // https://developer.apple.com/documentation/xcode-release-notes/xcode-14-release-notes
          BOOL isSandboxApp = YES;
          GGLog(@"is sandbox app is true");
#else
          BOOL isSandboxApp = [self FIRMessagingIsSandboxApp];
          GGLog(@"is sandbox app is %@",isSandboxApp?@"true":@"false");
#endif
    }

    return YES;
}

-(BOOL) FIRMessagingIsSandboxApp {
  static BOOL isSandboxApp = YES;
  static dispatch_once_t onceToken;
  dispatch_once(&onceToken, ^{
    isSandboxApp = ![self FIRMessagingIsProductionApp];
  });
  return isSandboxApp;
}


 -(BOOL) FIRMessagingIsProductionApp {
  const BOOL defaultAppTypeProd = YES;

  NSError *error = nil;
  if ([GULAppEnvironmentUtil isSimulator]) {
//    FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014,
//                            @"Running InstanceID on a simulator doesn't have APNS. "
//                            @"Use prod profile by default.");
    return defaultAppTypeProd;
  }

  if ([GULAppEnvironmentUtil isFromAppStore]) {
    // Apps distributed via AppStore or TestFlight use the Production APNS certificates.
    return defaultAppTypeProd;
  }
#if TARGET_OS_OSX || TARGET_OS_MACCATALYST
  NSString *path = [[[[NSBundle mainBundle] resourcePath] stringByDeletingLastPathComponent]
      stringByAppendingPathComponent:@"embedded.provisionprofile"];
#elif TARGET_OS_IOS || TARGET_OS_TV || TARGET_OS_WATCH
  NSString *path = [[[NSBundle mainBundle] bundlePath]
      stringByAppendingPathComponent:@"embedded.mobileprovision"];
#endif

  if ([GULAppEnvironmentUtil isAppStoreReceiptSandbox] && !path.length) {
    // Distributed via TestFlight
    return defaultAppTypeProd;
  }

  NSMutableData *profileData = [NSMutableData dataWithContentsOfFile:path options:0 error:&error];

  if (!profileData.length || error) {
//    NSString *errorString =
//        [NSString stringWithFormat:@"Error while reading embedded mobileprovision %@", error];
   // FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014, @"%@", errorString);
    return defaultAppTypeProd;
  }

  // The "embedded.mobileprovision" sometimes contains characters with value 0, which signals the
  // end of a c-string and halts the ASCII parser, or with value > 127, which violates strict 7-bit
  // ASCII. Replace any 0s or invalid characters in the input.
  uint8_t *profileBytes = (uint8_t *)profileData.bytes;
  for (int i = 0; i < profileData.length; i++) {
    uint8_t currentByte = profileBytes[i];
    if (!currentByte || currentByte > 127) {
      profileBytes[i] = '.';
    }
  }

  NSString *embeddedProfile = [[NSString alloc] initWithBytesNoCopy:profileBytes
                                                             length:profileData.length
                                                           encoding:NSASCIIStringEncoding
                                                       freeWhenDone:NO];

  if (error || !embeddedProfile.length) {
//    NSString *errorString =
//        [NSString stringWithFormat:@"Error while reading embedded mobileprovision %@", error];
//    FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014, @"%@", errorString);
    return defaultAppTypeProd;
  }

  NSScanner *scanner = [NSScanner scannerWithString:embeddedProfile];
  NSString *plistContents;
  if ([scanner scanUpToString:@"<plist" intoString:nil]) {
    if ([scanner scanUpToString:@"</plist>" intoString:&plistContents]) {
      plistContents = [plistContents stringByAppendingString:@"</plist>"];
    }
  }

  if (!plistContents.length) {
    return defaultAppTypeProd;
  }

  NSData *data = [plistContents dataUsingEncoding:NSUTF8StringEncoding];
  if (!data.length) {
//    FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014,
//                            @"Couldn't read plist fetched from embedded mobileprovision");
    return defaultAppTypeProd;
  }

  NSError *plistMapError;
  id plistData = [NSPropertyListSerialization propertyListWithData:data
                                                           options:NSPropertyListImmutable
                                                            format:nil
                                                             error:&plistMapError];
  if (plistMapError || ![plistData isKindOfClass:[NSDictionary class]]) {
    //NSString *errorString =
        [NSString stringWithFormat:@"Error while converting assumed plist to dict %@",
                                   plistMapError.localizedDescription];
    //FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014, @"%@", errorString);
    return defaultAppTypeProd;
  }
  NSDictionary *plistMap = (NSDictionary *)plistData;

  if ([plistMap valueForKeyPath:@"ProvisionedDevices"]) {
//    FIRMessagingLoggerDebug(kFIRMessagingMessageCodeInstanceID012,
//                            @"Provisioning profile has specifically provisioned devices, "
//                            @"most likely a Dev profile.");
  }

  NSString *apsEnvironment = [plistMap valueForKeyPath:kEntitlementsAPSEnvironmentKey];
  //NSString *debugString __unused =
      [NSString stringWithFormat:@"APNS Environment in profile: %@", apsEnvironment];
//  FIRMessagingLoggerDebug(kFIRMessagingMessageCodeInstanceID013, @"%@", debugString);

  // No aps-environment in the profile.
  if (!apsEnvironment.length) {
//    FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014,
//                            @"No aps-environment set. If testing on a device APNS is not "
//                            @"correctly configured. Please recheck your provisioning "
//                            @"profiles. If testing on a simulator this is fine since APNS "
//                            @"doesn't work on the simulator.");
    return defaultAppTypeProd;
  }

  if ([apsEnvironment isEqualToString:kAPSEnvironmentDevelopmentValue]) {
    return NO;
  }

  return defaultAppTypeProd;
}




- (BOOL)application:(UIApplication *)application continueUserActivity:(NSUserActivity *)userActivity restorationHandler:(void (^)(NSArray<id<UIUserActivityRestoring>> * _Nullable))restorationHandler{
    INInteraction *interaction = userActivity.interaction;
    INIntent *intent = interaction.intent;
    if([userActivity.activityType isEqual:@"INStartVideoCallIntent"]){
        INPerson *person = [(INStartAudioCallIntent *)intent contacts][0];
        CXHandle *handle = [[CXHandle alloc] initWithType:(CXHandleType)person.personHandle.type value:person.personHandle.value];
        
        UIAlertController *alertView = [UIAlertController alertControllerWithTitle:@"tips" message:[NSString stringWithFormat:@"用户活动类型为 INStartVideoCallIntent，切到视频,用户的用户信息为:%@",handle.value] preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {            
        }];
        [alertView addAction:action];
        [[UIViewControllerCJHelper findCurrentShowingViewController] presentViewController:alertView animated:YES completion:nil];
        return true;
    }
    return false;
}


-(void)onRegistered:(NSString *)Pushid{
    GGLog(@"pushid: %@",Pushid);
}


+(void)voipRegistration{
    GGLog(@"register voip token");
    dispatch_queue_t mainQueue = dispatch_get_main_queue();
    PKPushRegistry *voipRegistry = [[PKPushRegistry alloc] initWithQueue:mainQueue];
    [voipRegistry setDelegate:[CallKitManager getInstance]];
    NSMutableSet *desiredPushTypes = [[NSMutableSet alloc] init];
    [desiredPushTypes addObject:PKPushTypeVoIP];
    voipRegistry.desiredPushTypes = desiredPushTypes;
}


- (void)pushRegistry:(PKPushRegistry *)registry didUpdatePushCredentials: (PKPushCredentials *)credentials forType:(NSString *)type {
  // Register VoIP push token (a property of PKPushCredentials) with server
    NSLog(@"voip token:%@",[self getHexStringForData:credentials.token]); //应用启动获取token，并上传服务器
}


- (void)application:(UIApplication *)application
didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken {
#if TARGET_OS_SIMULATOR && TARGET_OS_IOS
          // If APNS token is available on iOS Simulator, we must use the sandbox profile
          // https://developer.apple.com/documentation/xcode-release-notes/xcode-14-release-notes
          BOOL isSandboxApp = YES;
          GGLog(@"is sandbox app is true");
    [[ZPNs shared] setDeviceToken:deviceToken isProduct:!isSandboxApp];
#else
          BOOL isSandboxApp = [self FIRMessagingIsSandboxApp];
          GGLog(@"is sandbox app is %@",isSandboxApp?@"true":@"false");
    [[ZPNs shared] setDeviceToken:deviceToken isProduct:!isSandboxApp];
#endif
    NSString *token = [self getHexStringForData:deviceToken];
    GGLog(@"[GGLog][ZPNs][deviceToken]:%@",token);
}

- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error API_AVAILABLE(ios(3.0)){
    GGLog(@"[GGLog][ZPNs] register failed!");
}


-(void)loadKeyCenter{

    NSUserDefaults *userDefault = [NSUserDefaults standardUserDefaults];
    if([userDefault objectForKey:@"appID"]!= nil){
        [KeyCenter setAppID:[[userDefault objectForKey:@"appID"] unsignedIntValue] ];
    }
    
    if([userDefault objectForKey:@"appSign"]!= nil){
        [KeyCenter setAppSign:[userDefault objectForKey:@"appSign"] ];
    }
    if([userDefault objectForKey:@"isUseToken"] != nil){
        [KeyCenter setIsUseToken:[[userDefault objectForKey:@"isUseToken"] boolValue]];
    }

    
}
- (NSString *)getHexStringForData:(NSData *)data
{
    if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 13) {
        
        if (![data isKindOfClass:[NSData class]]) {
            return @"";
        }
        NSUInteger len = [data length];
        char *chars = (char *)[data bytes];
        NSMutableString *hexString = [[NSMutableString alloc]init];
        for (NSUInteger i=0; i<len; i++) {
            [hexString appendString:[NSString stringWithFormat:@"%0.2hhx" , chars[i]]];
        }
        return hexString;
    } else {
         NSString *myToken = [[data description] stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"<>"]];
        myToken = [myToken stringByReplacingOccurrencesOfString:@" " withString:@""];
        return myToken;
    }
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler{
    
    GGLog(@"收到静默推送 %@",userInfo);
    completionHandler(UIBackgroundFetchResultNoData);
}

- (void)ZPNsNotificationCenter:(UNUserNotificationCenter *)center didReceiveNotificationResponse:(UNNotificationResponse *)response userInfo:(NSDictionary *)userInfo withCompletionHandler:(void (^)())completionHandler API_AVAILABLE(ios(10.0)){
    GGLog(@"Alert有效载荷 %@",response.notification.request.content.userInfo);
    
    
}

- (void)userNotificationCenter:(UNUserNotificationCenter *)center
       willPresentNotification:(UNNotification *)notification
         withCompletionHandler:(void (^)(UNNotificationPresentationOptions))completionHandler
    API_AVAILABLE(macos(10.14), ios(10.0)) {
    
    
}


- (void)redirectNSLogToDocumentFolder {
    // 如果已经连接Xcode调试则不输出到文件
//    if(isatty(STDOUT_FILENO)) {
//        return;
//    }
    
    UIDevice *device = [UIDevice currentDevice];
    if([[device model] hasSuffix:@"Simulator"]){ //在模拟器不保存到文件中
        return;
    }
    
    // 获取Document目录下的Log文件夹，若没有则新建
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *logDirectory = [[paths objectAtIndex:0] stringByAppendingPathComponent:@"Log"];
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL fileExists = [fileManager fileExistsAtPath:logDirectory];
    if (!fileExists) {
        [fileManager createDirectoryAtPath:logDirectory  withIntermediateDirectories:YES attributes:nil error:nil];
    }
    
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"zh_CN"]];
    [formatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"]; //每次启动后都保存一个新的日志文件中
    NSString *dateStr = [formatter stringFromDate:[NSDate date]];
    NSString *logFilePath = [logDirectory stringByAppendingFormat:@"/%@.txt",dateStr];
    
    // freopen 重定向输出输出流，将log输入到文件
    freopen([logFilePath cStringUsingEncoding:NSASCIIStringEncoding], "a+", stdout);
    freopen([logFilePath cStringUsingEncoding:NSASCIIStringEncoding], "a+", stderr);
}

@end
