//
//  CallKitManager.m
//  ZIMExampleDemo
//
//  Created by 武耀琳 on 2023/1/31.
//

#import "CallKitManager.h"
#import "ZGHUDHelper.h"
#import <ZPNs/ZPNs.h>
#import "ZGZIMManager.h"
#import "KeyCenter.h"
#import "NSDictionary+MyLog.h"
#import "GULAppEnvironmentUtil.h"
static CallKitManager *_callKitManager = nil;
#if TARGET_OS_IOS || TARGET_OS_TV || TARGET_OS_WATCH
static NSString *const kEntitlementsAPSEnvironmentKey = @"Entitlements.aps-environment";
#else
static NSString *const kEntitlementsAPSEnvironmentKey =
    @"Entitlements.com.apple.developer.aps-environment";
#endif
static NSString *const kAPSEnvironmentDevelopmentValue = @"development";


@interface CallKitManager()

@property CXProvider *cxProvider;

@property CXCallController *cxCallController;

@property NSString *callID;

@end

@implementation CallKitManager

+(CallKitManager *)getInstance{
    if(!_callKitManager){
        @synchronized (self) {
            if(!_callKitManager){
                _callKitManager = [[CallKitManager alloc] init];
                CXProviderConfiguration* configInternal = [[CXProviderConfiguration alloc] initWithLocalizedName:@""];
                configInternal.supportsVideo = true;
                configInternal.maximumCallsPerCallGroup = 1;
                configInternal.maximumCallGroups = 1;
                configInternal.supportedHandleTypes = [[NSSet alloc] initWithObjects:[NSNumber numberWithInt:CXHandleTypeGeneric],[NSNumber numberWithInt:CXHandleTypePhoneNumber], nil];
                UIImage* iconMaskImage = [UIImage imageNamed:@"yijian"];
                configInternal.iconTemplateImageData = UIImagePNGRepresentation(iconMaskImage);
                _callKitManager.cxProvider = [[CXProvider alloc] initWithConfiguration:configInternal];
                [_callKitManager.cxProvider setDelegate:_callKitManager queue:dispatch_get_main_queue()];
                _callKitManager.cxCallController = [[CXCallController alloc] initWithQueue:dispatch_get_main_queue()];
            }
        }
    }
    return _callKitManager;
}

-(void)reportIncomingCallWithTitle:(NSString *)title CallID:(NSString *)callID{
    CXCallUpdate* update = [[CXCallUpdate alloc] init];
    update.supportsDTMF = YES;
    update.supportsHolding = YES;
    update.supportsGrouping = YES;
    update.supportsUngrouping = YES;
    update.hasVideo = true;
    update.remoteHandle = [[CXHandle alloc] initWithType:CXHandleTypeGeneric value:@"my group"];
    update.localizedCallerName = title;
    NSUUID *uuid = [NSUUID UUID];
    self.callID = callID;
    
    //弹出电话页面
    [self.cxProvider reportNewIncomingCallWithUUID:uuid update:update completion:^(NSError * _Nullable error) {
        
    }];
}

- (void)pushRegistry:(nonnull PKPushRegistry *)registry didUpdatePushCredentials:(nonnull PKPushCredentials *)pushCredentials forType:(nonnull PKPushType)type {
#if TARGET_OS_SIMULATOR && TARGET_OS_IOS
          // If APNS token is available on iOS Simulator, we must use the sandbox profile
          // https://developer.apple.com/documentation/xcode-release-notes/xcode-14-release-notes
          BOOL isSandboxApp = YES;
          GGLog(@"is sandbox app is true");
            NSString *voipToken = [self getHexStringForData:[pushCredentials token] ];
            [[ZPNs shared] setVoipToken:[pushCredentials token] isProduct:!isSandboxApp];
            GGLog(@"[ZPNs] voip token:%@",voipToken);
            NSLog(@"voip token %@",voipToken);
#else
          BOOL isSandboxApp = [self FIRMessagingIsSandboxApp];
          GGLog(@"is sandbox app is %@",isSandboxApp?@"true":@"false");
        NSString *voipToken = [self getHexStringForData:[pushCredentials token] ];
        [[ZPNs shared] setVoipToken:[pushCredentials token] isProduct:!isSandboxApp];
        GGLog(@"[ZPNs] voip token:%@",voipToken);
        NSLog(@"voip token %@",voipToken);
#endif
}

- (void)pushRegistry:(PKPushRegistry *)registry didReceiveIncomingPushWithPayload:(PKPushPayload *)payload forType:(PKPushType)type withCompletionHandler:(void (^)(void))completion{
//    NSString *callID = [[[payload.dictionaryPayload objectForKey:@"aps"] objectForKey:@"payload"] objectForKey:@"callID"];
    NSString *callID = @"ZIM Moke";
    GGLog(@"VoIP有效载荷%@",payload.dictionaryPayload);
    NSLog(@"VoIP有效载荷 %@",payload.dictionaryPayload);
    NSString *title = [[[[payload dictionaryPayload] objectForKey:@"aps"] objectForKey:@"alert"] objectForKey:@"title"];
    [self reportIncomingCallWithTitle:title CallID:callID];
    completion();
}


//当接收到呼叫重置时 调用的函数，这个函数必须被实现，其不需做任何逻辑，只用来重置状态
- (void)providerDidReset:(CXProvider *)provider{
    
}
//呼叫开始时回调
- (void)providerDidBegin:(CXProvider *)provider{

}
//音频会话激活状态的回调
- (void)provider:(CXProvider *)provider didActivateAudioSession:(AVAudioSession *)audioSession{
    
}
//音频会话停用的回调
- (void)provider:(CXProvider *)provider didDeactivateAudioSession:(AVAudioSession *)audioSession{

}
//行为超时的回调
- (void)provider:(CXProvider *)provider timedOutPerformingAction:(CXAction *)action{
    [action fulfill];
}
//有事务被提交时调用
//如果返回YES 则表示事务被捕获处理 后面的回调都不会调用 如果返回NO 则表示事务不被捕获，会回调后面的函数
- (BOOL)provider:(CXProvider *)provider executeTransaction:(CXTransaction *)transaction{
    return NO;
}
//点击开始按钮的回调
- (void)provider:(CXProvider *)provider performStartCallAction:(CXStartCallAction *)action{
    [ZGHUDHelper showMessage:[NSString stringWithFormat:@"点击开始按钮回调"]];
    [action fulfill];
}
//点击接听按钮的回调
- (void)provider:(CXProvider *)provider performAnswerCallAction:(CXAnswerCallAction *)action{
    [action fulfill];
}
//点击结束按钮的回调
- (void)provider:(CXProvider *)provider performEndCallAction:(CXEndCallAction *)action{
    [action fulfill];
}
//点击保持通话按钮的回调
- (void)provider:(CXProvider *)provider performSetHeldCallAction:(CXSetHeldCallAction *)action{
    [action fulfill];
}
//点击静音按钮的回调
- (void)provider:(CXProvider *)provider performSetMutedCallAction:(CXSetMutedCallAction *)action{
    [action fulfill];
}
//点击组按钮的回调
- (void)provider:(CXProvider *)provider performSetGroupCallAction:(CXSetGroupCallAction *)action{
    [action fulfill];
}
//DTMF功能回调
- (void)provider:(CXProvider *)provider performPlayDTMFCallAction:(CXPlayDTMFCallAction *)action{
    [action fulfill];
}


- (NSString *)getHexStringForData:(NSData *)data
{
    if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 13) {
        if (![data isKindOfClass:[NSData class]]) {
            return @"";
        }
        NSUInteger len = [data length];
        char *chars = (char *)[data bytes];
        NSMutableString *hexString = [[NSMutableString alloc]init];
        for (NSUInteger i=0; i<len; i++) {
            [hexString appendString:[NSString stringWithFormat:@"%0.2hhx" , chars[i]]];
        }
        return hexString;
    } else {
         NSString *myToken = [[data description] stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"<>"]];
        myToken = [myToken stringByReplacingOccurrencesOfString:@" " withString:@""];
        return myToken;
    }
}

-(BOOL) FIRMessagingIsSandboxApp {
  static BOOL isSandboxApp = YES;
  static dispatch_once_t onceToken;
  dispatch_once(&onceToken, ^{
    isSandboxApp = ![self FIRMessagingIsProductionApp];
  });
  return isSandboxApp;
}


 -(BOOL) FIRMessagingIsProductionApp {
  const BOOL defaultAppTypeProd = YES;

  NSError *error = nil;
  if ([GULAppEnvironmentUtil isSimulator]) {
//    FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014,
//                            @"Running InstanceID on a simulator doesn't have APNS. "
//                            @"Use prod profile by default.");
    return defaultAppTypeProd;
  }

  if ([GULAppEnvironmentUtil isFromAppStore]) {
    // Apps distributed via AppStore or TestFlight use the Production APNS certificates.
    return defaultAppTypeProd;
  }
#if TARGET_OS_OSX || TARGET_OS_MACCATALYST
  NSString *path = [[[[NSBundle mainBundle] resourcePath] stringByDeletingLastPathComponent]
      stringByAppendingPathComponent:@"embedded.provisionprofile"];
#elif TARGET_OS_IOS || TARGET_OS_TV || TARGET_OS_WATCH
  NSString *path = [[[NSBundle mainBundle] bundlePath]
      stringByAppendingPathComponent:@"embedded.mobileprovision"];
#endif

  if ([GULAppEnvironmentUtil isAppStoreReceiptSandbox] && !path.length) {
    // Distributed via TestFlight
    return defaultAppTypeProd;
  }

  NSMutableData *profileData = [NSMutableData dataWithContentsOfFile:path options:0 error:&error];

  if (!profileData.length || error) {
//    NSString *errorString =
//        [NSString stringWithFormat:@"Error while reading embedded mobileprovision %@", error];
   // FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014, @"%@", errorString);
    return defaultAppTypeProd;
  }

  // The "embedded.mobileprovision" sometimes contains characters with value 0, which signals the
  // end of a c-string and halts the ASCII parser, or with value > 127, which violates strict 7-bit
  // ASCII. Replace any 0s or invalid characters in the input.
  uint8_t *profileBytes = (uint8_t *)profileData.bytes;
  for (int i = 0; i < profileData.length; i++) {
    uint8_t currentByte = profileBytes[i];
    if (!currentByte || currentByte > 127) {
      profileBytes[i] = '.';
    }
  }

  NSString *embeddedProfile = [[NSString alloc] initWithBytesNoCopy:profileBytes
                                                             length:profileData.length
                                                           encoding:NSASCIIStringEncoding
                                                       freeWhenDone:NO];

  if (error || !embeddedProfile.length) {
//    NSString *errorString =
//        [NSString stringWithFormat:@"Error while reading embedded mobileprovision %@", error];
//    FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014, @"%@", errorString);
    return defaultAppTypeProd;
  }

  NSScanner *scanner = [NSScanner scannerWithString:embeddedProfile];
  NSString *plistContents;
  if ([scanner scanUpToString:@"<plist" intoString:nil]) {
    if ([scanner scanUpToString:@"</plist>" intoString:&plistContents]) {
      plistContents = [plistContents stringByAppendingString:@"</plist>"];
    }
  }

  if (!plistContents.length) {
    return defaultAppTypeProd;
  }

  NSData *data = [plistContents dataUsingEncoding:NSUTF8StringEncoding];
  if (!data.length) {
//    FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014,
//                            @"Couldn't read plist fetched from embedded mobileprovision");
    return defaultAppTypeProd;
  }

  NSError *plistMapError;
  id plistData = [NSPropertyListSerialization propertyListWithData:data
                                                           options:NSPropertyListImmutable
                                                            format:nil
                                                             error:&plistMapError];
  if (plistMapError || ![plistData isKindOfClass:[NSDictionary class]]) {
    //NSString *errorString =
        [NSString stringWithFormat:@"Error while converting assumed plist to dict %@",
                                   plistMapError.localizedDescription];
    //FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014, @"%@", errorString);
    return defaultAppTypeProd;
  }
  NSDictionary *plistMap = (NSDictionary *)plistData;

  if ([plistMap valueForKeyPath:@"ProvisionedDevices"]) {
//    FIRMessagingLoggerDebug(kFIRMessagingMessageCodeInstanceID012,
//                            @"Provisioning profile has specifically provisioned devices, "
//                            @"most likely a Dev profile.");
  }

  NSString *apsEnvironment = [plistMap valueForKeyPath:kEntitlementsAPSEnvironmentKey];
  //NSString *debugString __unused =
      [NSString stringWithFormat:@"APNS Environment in profile: %@", apsEnvironment];
//  FIRMessagingLoggerDebug(kFIRMessagingMessageCodeInstanceID013, @"%@", debugString);

  // No aps-environment in the profile.
  if (!apsEnvironment.length) {
//    FIRMessagingLoggerError(kFIRMessagingMessageCodeInstanceID014,
//                            @"No aps-environment set. If testing on a device APNS is not "
//                            @"correctly configured. Please recheck your provisioning "
//                            @"profiles. If testing on a simulator this is fine since APNS "
//                            @"doesn't work on the simulator.");
    return defaultAppTypeProd;
  }

  if ([apsEnvironment isEqualToString:kAPSEnvironmentDevelopmentValue]) {
    return NO;
  }

  return defaultAppTypeProd;
}



@end
