//
//  ConversationListDataSource.m
//  ZIMExampleDemo
//
//  Created by 武耀琳 on 2022/1/19.
//

#import "ConversationListDataSource.h"
#import "ZGZIMManager.h"

static ConversationListDataSource *dataSource = nil;
@interface ConversationListDataSource()<ZIMEventDelegate>

@property(nonatomic,weak)id<conversationSourceNotifcation> delegate;

@property ZIMConversation *nextConversation;


@property NSString *unReadMessageCount;
@end
@implementation ConversationListDataSource
+(ConversationListDataSource *)shared{
    if(dataSource == nil){
        dataSource = [[ConversationListDataSource alloc] init];
        [[ZGZIMManager shared] addZIMEventDelegate:(id<ZIMEventDelegate>)dataSource];
        dataSource.unReadMessageCount = nil;
    }
    
    return dataSource;
}
+(void)releaseMySelf{
    dataSource = nil;
}
-(void)queryMoreConversationList{
    ZIMConversationQueryConfig *queryConfig = [[ZIMConversationQueryConfig alloc] init];
    queryConfig.count = 20;
    if(self.conversationList.count!=0 && self.conversationList != nil){
        queryConfig.nextConversation = self.conversationList[self.conversationList.count -1];
    }
    [[ZGZIMManager shared] queryConversationListWithConfig:queryConfig callback:^(NSArray<ZIMConversation *> * _Nonnull conversationList, ZIMError * _Nonnull errorInfo) {
        if(self.conversationList == nil){
            self.conversationList = (NSMutableArray *)conversationList;
            
        }else{
            for (ZIMConversation *conversation in conversationList) {
                [self.conversationList addObject:conversation];
            }
        }
        [self conversationUpdate];
    }];
}
-(void)deleteConversation:(int)Index{
    [self.conversationList removeObjectAtIndex:Index];
    [self conversationUpdate];
}


-(void)deleteConversationByConversationID:(NSString *)conversationID{
    ZIMConversation *selectedConver;
    for (ZIMConversation *conversation in self.conversationList) {
        if([conversation.conversationID isEqual:conversationID]){
            selectedConver = conversation;
        }
    }
    [self.conversationList removeObject:selectedConver];
}

-(void)setConversationSourceNotifcationDelegate:(id<conversationSourceNotifcation>)delegate{
    self.delegate = delegate;
}
-(void)conversationSelected:(NSString *)conversationID
           conversationType:(int)type{
    dispatch_async(dispatch_get_main_queue(), ^{
        
        if([self.delegate respondsToSelector:@selector(conversationSelectedNotfication:conversationType:)]){
            [self.delegate conversationSelectedNotfication:conversationID conversationType:type];
        }
    });
}
- (void)conversationChanged:(NSArray<ZIMConversationChangeInfo *> *)conversationChangeInfoList{
    for (ZIMConversationChangeInfo *changedInfo in conversationChangeInfoList) {
        if(changedInfo.event == 0){
            //增
            [self.conversationList insertObject:changedInfo.conversation atIndex:0];
        }
        else if (changedInfo.event == 1){
            //改
            ZIMConversation *targetConv;
            for (ZIMConversation *myConversation in self.conversationList) {
                if([myConversation.conversationID isEqual:changedInfo.conversation.conversationID]){
                    targetConv = myConversation;
                    break;
                }
            }
            [self.conversationList removeObject:targetConv];
            //[self.conversationList addObject:changedInfo.conversation];
            [self.conversationList insertObject:changedInfo.conversation atIndex:0];
        }
        else if (changedInfo.event == 2){
            //删
            ZIMConversation *targetConv;
            for (ZIMConversation *myConversation in self.conversationList) {
                if([myConversation.conversationID isEqual:changedInfo.conversation.conversationID]){
                    targetConv = myConversation;
                    break;
                }
            }
        }
    }
    [self conversationUpdate];
}

-(void)conversationUpdate{
    dispatch_async(dispatch_get_main_queue(), ^{
        
        if([self.delegate respondsToSelector:@selector(conversationUpdateNotfication)]){
            [self.delegate conversationUpdateNotfication];
        }
    });
}

- (void)conversationTotalUnreadMessageCountUpdated:(unsigned int)totalUnreadMessageCount{
    if(100>totalUnreadMessageCount >0){
        self.unReadMessageCount = [NSString stringWithFormat:@"%u",totalUnreadMessageCount];
    }else if(totalUnreadMessageCount == 0){
        self.unReadMessageCount = nil;
    }else if (totalUnreadMessageCount>=100){
        self.unReadMessageCount =@"99+";
    }
    dispatch_async(dispatch_get_main_queue(), ^{
        
        if([self.delegate respondsToSelector:@selector(unreadCountupdateNotification)]){
            [self.delegate unreadCountupdateNotification];
        }
    });
}

-(NSString *)readUnreadMessageCount{
    return _unReadMessageCount;
}
@end
