//
//  ConversationListViewController.m
//  ZIMExampleDemo
//
//  Created by 武耀琳 on 2022/1/19.
//

#import "ConversationListViewController.h"
#import "ConversationTableViewCell.h"
#import "ZGPopMenuView.h"
#import "CreatePeerViewController.h"
#import "CreateGroupViewController.h"
#import "JoinGroupViewController.h"
#import "ZGZIMManager.h"
#import "LoginViewController.h"
#import "ConversationListDataSource.h"
#import "PeerViewController.h"
#import "GroupViewController.h"
#import "CreateRoomViewController.h"
#import "JoinRoomViewController.h"
#import "MJRefresh.h"
@interface ConversationListViewController ()<UITableViewDelegate,UITableViewDataSource,conversationSourceNotifcation,ZIMEventDelegate>
@property (weak, nonatomic) IBOutlet UITableView *conversationListTableView;

@property NSMutableArray *conversationIDList;
@property NSMutableArray *conversationTitleList;
@property NSMutableArray *conversationContentList;
@property NSMutableArray *conversationTimeList;
@property NSMutableArray *conversationUnreadNumberList;
@property NSMutableArray *conversationTypeList;
@property NSMutableArray *conversationAvatarList;
@property (weak, nonatomic) IBOutlet UIButton *setButton;
@property (weak, nonatomic) IBOutlet UIButton *backButton;
@end

@implementation ConversationListViewController
//MARK: - common

- (instancetype)init
{
    self = [super init];
    if (self) {
       
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.setButton setTitle:@"" forState:UIControlStateNormal];
    [self.backButton setTitle:@"" forState:UIControlStateNormal];
    [self initConversationList];
    [[ZGZIMManager shared] addZIMEventDelegate:self];
    self.conversationListTableView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
    [[ConversationListDataSource shared] setConversationSourceNotifcationDelegate:self];
    self.conversationListTableView.mj_footer = [MJRefreshBackNormalFooter footerWithRefreshingBlock:^{
        [[ConversationListDataSource shared] queryMoreConversationList];
        [self.conversationListTableView.mj_footer endRefreshing];
    }];
    if ([[[ConversationListDataSource shared] readUnreadMessageCount] isEqual:@"0"] ||[[ConversationListDataSource shared] readUnreadMessageCount] ==nil) {
        [self.tabBarItem setBadgeValue:nil];
    }else{
        [self.tabBarItem setBadgeValue:[[ConversationListDataSource shared] readUnreadMessageCount]];
    }
    
}

//
//-(void)viewWillDisappear:(BOOL)animated{
//    [super viewWillDisappear:animated];
//    [[ZIM getInstance] logout];
//}



-(void)addConversationList:(nonnull NSString *)title
                   content:(nonnull NSString *)content
                      time:(nonnull NSNumber *)time
            conversationID:(nonnull NSString *)conversationID
              unreadNumber:(nonnull NSNumber *)unreadNumber
          conversationType:(nonnull NSNumber *)conversationType
                    avatar:(nonnull NSString *)avatar{
    
    [_conversationTitleList addObject:title];
    [_conversationContentList addObject:content];
    [_conversationTimeList addObject:time];
    [_conversationIDList addObject:conversationID];
    [_conversationAvatarList addObject:avatar];
    [_conversationUnreadNumberList addObject:unreadNumber];
    [_conversationTypeList addObject:conversationType];
}

- (void)conversationTotalUnreadMessageCountUpdated:(unsigned int)totalUnreadMessageCount{
    NSString *unReadCount;
    if(totalUnreadMessageCount >0 &&totalUnreadMessageCount<1000){
        unReadCount = [NSString stringWithFormat:@"%u",totalUnreadMessageCount];
    }else if(totalUnreadMessageCount == 0){
        unReadCount = nil;
    }else if (totalUnreadMessageCount>=1000){
        unReadCount =@"999+";
    }
    if([unReadCount isEqual:@"0"]){
        [self.tabBarItem setBadgeValue:nil];
    }else{
        self.tabBarItem.badgeValue = unReadCount;
    }
}
//MARK: - TableViewDelegate
- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _conversationTimeList.count;
}
- (nonnull UITableViewCell *)tableView:(nonnull UITableView *)tableView cellForRowAtIndexPath:(nonnull NSIndexPath *)indexPath {
    ConversationTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"ConversationTableViewCell"];
    [cell.titleLabel setText:_conversationTitleList[indexPath.row]];
    [cell.contentLabel setText:_conversationContentList[indexPath.row]];
    NSString *dateStr = [_conversationTimeList[indexPath.row] longLongValue]!=0?[self getDateDisplayString:[_conversationTimeList[indexPath.row] longLongValue]]:@"";
    [cell.timeLabel setText:dateStr];
    [cell.unReadNumberLabel setText:[_conversationUnreadNumberList[indexPath.row] stringValue]];
    
    if([_conversationTypeList[indexPath.row] intValue] == 0){
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
            UIImage *avatarImage;
            if(![self.conversationAvatarList[indexPath.row] isEqual:@""]){
                avatarImage = [UIImage imageWithData:[NSData dataWithContentsOfURL:[NSURL URLWithString:self.conversationAvatarList[indexPath.row]]]];
            }else{
                avatarImage = [UIImage imageNamed:@"people_fill"];
            }
            dispatch_async(dispatch_get_main_queue(), ^{
                [cell.personImageView setImage:avatarImage];
            });
        });
    } else if([_conversationTypeList[indexPath.row] intValue] == 1){
        [cell.personImageView setImage:[UIImage imageNamed:@"md-room"]];
    } else if([_conversationTypeList[indexPath.row] intValue] == 2){
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
            UIImage *avatarImage;
            if(![self.conversationAvatarList[indexPath.row] isEqual:@""]){
                avatarImage = [UIImage imageWithData:[NSData dataWithContentsOfURL:[NSURL URLWithString:self.conversationAvatarList[indexPath.row]]]];
            }else{
                avatarImage = [UIImage imageNamed:@"people"];
            }
            dispatch_async(dispatch_get_main_queue(), ^{
                [cell.personImageView setImage:avatarImage];
            });
        });
    }
    
    if([_conversationUnreadNumberList[indexPath.row] intValue] == 0){
        cell.unReadNumberLabel.hidden = true;
        cell.unReadNumberImageView.hidden = true;
    }else{
        cell.unReadNumberLabel.hidden = false;
        cell.unReadNumberImageView.hidden = false;
    }
    cell.conversationID = _conversationIDList[indexPath.row];
    cell.conversationType = [_conversationTypeList[indexPath.row] intValue];
    
    return cell;
}

-(UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath{

 return UITableViewCellEditingStyleDelete;//手势滑动删除
}


-(void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    editingStyle = UITableViewCellEditingStyleDelete;
  if (editingStyle ==UITableViewCellEditingStyleDelete) {
      ZIMConversationDeleteConfig *deleteConfig = [[ZIMConversationDeleteConfig alloc] init];
      deleteConfig.isAlsoDeleteServerConversation = true;
      [[ZGZIMManager shared] deleteConversation:self.conversationIDList[indexPath.row] type:[self.conversationTypeList[indexPath.row] intValue] config:deleteConfig callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {
          if(errorInfo.code == 0){
              [[ConversationListDataSource shared] deleteConversation:(int)indexPath.row];
          }
      }];
   }
}
- (NSString *)tableView:(UITableView *)tableView titleForDeleteConfirmationButtonForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    return NSLocalizedString(@"Delete",nil);
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}
//MARK: -
-(void)initConversationList{
    _conversationAvatarList = [[NSMutableArray alloc] init];
    _conversationIDList = [[NSMutableArray alloc] init];
    _conversationTitleList = [[NSMutableArray alloc] init];
    _conversationContentList = [[NSMutableArray alloc] init];
    _conversationTimeList = [[NSMutableArray alloc] init];
    _conversationUnreadNumberList = [[NSMutableArray alloc] init];
    _conversationTypeList = [[NSMutableArray alloc] init];
    for(ZIMConversation *conversation in [ConversationListDataSource shared].conversationList){
        NSString *msgContent = [[NSString alloc] init];
        if(conversation.lastMessage.type == ZIMMessageTypeText){
            msgContent = ((ZIMTextMessage *)conversation.lastMessage).message;
        }
        else if (conversation.lastMessage.type == ZIMMessageTypeImage){
            msgContent = [NSString stringWithFormat:@"[%@]",NSLocalizedString(@"image", nil)];
        }
        else if (conversation.lastMessage.type == ZIMMessageTypeFile){
            msgContent = [NSString stringWithFormat:@"[%@]",NSLocalizedString(@"file", nil)];
        }
        else if (conversation.lastMessage.type == ZIMMessageTypeVideo){
            msgContent = [NSString stringWithFormat:@"[%@]",NSLocalizedString(@"video", nil)];
        }
        else if (conversation.lastMessage.type == ZIMMessageTypeAudio){
            msgContent = [NSString stringWithFormat:@"[%@]",NSLocalizedString(@"audio", nil)];
        }
        else if (conversation.lastMessage.type == ZIMMessageTypeRevoke){
            msgContent = [NSString stringWithFormat:@"%@ %@",((ZIMRevokeMessage *)conversation.lastMessage).operatedUserID,NSLocalizedString(@"has revoked the message", nil)];
        }
        else{
            msgContent = @"[未知消息类型]";
        }
        [self addConversationList:(conversation.conversationName!=nil &&![conversation.conversationName isEqual:@"" ])?conversation.conversationName:conversation.conversationID content:msgContent
                    time:[NSNumber numberWithUnsignedLongLong:conversation.lastMessage.timestamp]
                   conversationID:conversation.conversationID unreadNumber:[NSNumber numberWithUnsignedInt:conversation.unreadMessageCount] conversationType:[NSNumber numberWithInt:(int)conversation.type] avatar:conversation.conversationAvatarUrl];
    }
    [_conversationListTableView reloadData];
}


-(void)conversationSelectedNotfication:(NSString *)conversationID conversationType:(int)type{
    ZIMMessageQueryConfig *config = [[ZIMMessageQueryConfig alloc] init];
    config.count = 20;
    config.reverse = true;
    if(type == 0){
        PeerViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"PeerViewController"];
        vc.toUserID = conversationID;
        [self.navigationController pushViewController:vc animated:YES];
    }
    else if (type == 1){
        //进入房间
    }
    else if (type == 2){
        //进入群聊
        GroupViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"GroupViewController"];

        vc.toGroupID = conversationID;
        [self.navigationController pushViewController:vc animated:YES];
    }
}
//MARK: -TIMETOOLS
//- (NSString *) compareCurrentTime:(NSDate *)date
//{
//
//NSDate *timeDate = date;
//
//NSDate *currentDate = [NSDate date];
//
//NSTimeInterval timeInterval = [currentDate timeIntervalSinceDate:timeDate];
//
//long temp = 0;
//
//NSString *result;
//
//if (timeInterval/60 < 1) {
//
//result = [NSString stringWithFormat:@"刚刚"];
//
//}
//
//else if((temp = timeInterval/60) <60){
//
//result = [NSString stringWithFormat:@"%ld分钟前",temp];
//
//}
//
//else if((temp = temp/60) <24){
//
//result = [NSString stringWithFormat:@"%ld小时前",temp];
//
//}
//
//else if((temp = temp/24) <30){
//
//result = [NSString stringWithFormat:@"%ld天前",temp];
//
//}
//
//else if((temp = temp/30) <12){
//
//result = [NSString stringWithFormat:@"%ld月前",temp];
//
//}
//
//else{
//
//temp = temp/12;
//
//result = [NSString stringWithFormat:@"%ld年前",temp];
//
//}
//
//return  result;
//
//}
//时间显示内容
-(NSString *)getDateDisplayString:(long long) miliSeconds{
    NSLog(@"-时间戳---%lld_----",miliSeconds);

    NSTimeInterval tempMilli = miliSeconds;
    NSTimeInterval seconds = tempMilli/1000.0;
    NSDate *myDate = [NSDate dateWithTimeIntervalSince1970:seconds];
    
    NSCalendar *calendar = [ NSCalendar currentCalendar ];
    int unit = NSCalendarUnitDay | NSCalendarUnitMonth |  NSCalendarUnitYear ;
    NSDateComponents *nowCmps = [calendar components:unit fromDate:[ NSDate date ]];
    NSDateComponents *myCmps = [calendar components:unit fromDate:myDate];
    
    NSDateFormatter *dateFmt = [[NSDateFormatter alloc ] init ];
    
    //2. 指定日历对象,要去取日期对象的那些部分.
    NSDateComponents *comp =  [calendar components:NSCalendarUnitYear|NSCalendarUnitMonth|NSCalendarUnitDay|NSCalendarUnitWeekday fromDate:myDate];
    
    if (nowCmps.year != myCmps.year) {
        dateFmt.dateFormat = @"yyyy-MM-dd hh:mm";
    } else {
        if (nowCmps.day==myCmps.day) {
            
            dateFmt.AMSymbol = NSLocalizedString(@"AM",nil);
            dateFmt.PMSymbol = NSLocalizedString(@"PM",nil);
            dateFmt.dateFormat = @"aaa hh:mm";
            
        } else if((nowCmps.day-myCmps.day)==1) {
            dateFmt.dateFormat = NSLocalizedString(@"yesterday",nil);;
        } else {
            if ((nowCmps.day-myCmps.day) <=7) {
                switch (comp.weekday) {
                    case 1:
                        dateFmt.dateFormat = NSLocalizedString(@"Weekday",nil);
                        break;
                    case 2:
                        dateFmt.dateFormat = NSLocalizedString(@"Monday",nil);
                        break;
                    case 3:
                        dateFmt.dateFormat = NSLocalizedString(@"Tuesday",nil);
                        break;
                    case 4:
                        dateFmt.dateFormat = NSLocalizedString(@"Wednesday",nil);
                        break;
                    case 5:
                        dateFmt.dateFormat = NSLocalizedString(@"Thursday",nil);
                        break;
                    case 6:
                        dateFmt.dateFormat = NSLocalizedString(@"Friday",nil);
                        break;
                    case 7:
                        dateFmt.dateFormat = NSLocalizedString(@"Saturday",nil);
                        break;
                    default:
                        break;
                }
            }else {
                dateFmt.dateFormat = @"MM-dd hh:mm";
            }
        }
    }
    return [dateFmt stringFromDate:myDate];
}
-(void)conversationUpdateNotfication{
    [self initConversationList];
}

-(void)unreadCountupdateNotification{
    
}
@end
