//
//  GlobalSearchViewController.m
//  ZIMExampleDemo
//
//  Created by lizhanpeng on 2023/6/28.
//

#import "GlobalSearchViewController.h"
#import "GlobalMessageSearchedListViewController.h"
#import <ZIM/ZIM.h>

@interface GlobalSearchViewController()

@property (weak, nonatomic) IBOutlet UITextField *keywordsInputTextField;
@property (weak, nonatomic) IBOutlet UITextField *senderUserIDInputTextField;
@property (weak, nonatomic) IBOutlet UISwitch *textMsgTypeSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *fileMsgTypeSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *imgMsgTypeSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *audioMsgTypeSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *videoMsgTypeSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *customMsgTypeSwitch;

@property (weak, nonatomic) IBOutlet UISegmentedControl *globalSearchType;

@property (weak, nonatomic) IBOutlet UIDatePicker *fromSentTimeDate;
@property (weak, nonatomic) IBOutlet UIDatePicker *toSentTimeDate;

@end

@implementation GlobalSearchViewController


- (void)viewDidLoad {
    [super viewDidLoad];
    self.navigationItem.title = self.viewTitle;
}


- (IBAction)onSearchButtonClicked:(id)sender {
    if(self.globalSearchType.selectedSegmentIndex == 0) {
        
        ZIMConversationSearchConfig *config = [[ZIMConversationSearchConfig alloc] init];
        
        
       // ZIMConversationMessageGlobalSearchConfig *config = [[ZIMConversationMessageGlobalSearchConfig alloc] init];
        
        if(self.keywordsInputTextField.text.length > 0) {
            config.keywords = [self.keywordsInputTextField.text componentsSeparatedByString:@" "];
        }
        
        if(self.senderUserIDInputTextField.text.length > 0) {
            config.senderUserIDs = [self.senderUserIDInputTextField.text componentsSeparatedByString:@" "];
        }
        
        config.totalConversationCount = 10;
        config.conversationMessageCount = 10;
        config.nextFlag = 0;
        //config.reverse = NO;
        config.startTime = [[self.fromSentTimeDate.calendar startOfDayForDate:self.fromSentTimeDate.date] timeIntervalSince1970] * 1000;
        config.endTime = ([[self.fromSentTimeDate.calendar startOfDayForDate:self.fromSentTimeDate.date] timeIntervalSince1970] + 86400) * 1000;
        NSMutableArray *msgTypes = [NSMutableArray array];
        if(self.textMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeText)];
        }
        if(self.fileMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeFile)];
        }
        if(self.imgMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeImage)];
        }
        if(self.audioMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeAudio)];
        }
        if(self.videoMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeVideo)];
        }
        if(self.customMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeCustom)];
        }
        config.messageTypes = msgTypes;
        
        [[ZIM getInstance] searchLocalConversationsWithConfig:config callback:^(NSArray<ZIMConversationSearchInfo *> * _Nonnull conversationSearchInfoList, unsigned int nextFlag, ZIMError * _Nonnull errorInfo) {
            if(errorInfo.code == ZIMErrorCodeSuccess) {
                GlobalMessageSearchedListViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"GlobalMessageSearchedViewController"];
                vc.searchInInfoList = conversationSearchInfoList;
                [self.navigationController pushViewController:vc animated:YES];
            }
        }];
    } else {
        ZIMMessageSearchConfig *config = [[ZIMMessageSearchConfig alloc] init];
        if(self.keywordsInputTextField.text.length > 0) {
            config.keywords = [self.keywordsInputTextField.text componentsSeparatedByString:@" "];
        }
        
        if(self.senderUserIDInputTextField.text.length > 0) {
            config.senderUserIDs = [self.senderUserIDInputTextField.text componentsSeparatedByString:@" "];
        }
        
        config.count = 5;
        config.nextMessage = nil;
        config.order = ZIMMessageOrderDescending;
        config.startTime = [self.fromSentTimeDate.date timeIntervalSince1970] * 1000;
        config.endTime = [self.toSentTimeDate.date timeIntervalSince1970] * 1000;
        NSMutableArray *msgTypes = [NSMutableArray array];
        if(self.textMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeText)];
        }
        if(self.fileMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeFile)];
        }
        if(self.imgMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeImage)];
        }
        if(self.audioMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeAudio)];
        }
        if(self.videoMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeVideo)];
        }
        if(self.customMsgTypeSwitch.on) {
            [msgTypes addObject:@(ZIMMessageTypeCustom)];
        }
        config.messageTypes = msgTypes;
        
        [[ZIM getInstance] searchGlobalLocalMessagesWithConfig:config callback:^(NSArray<ZIMMessage *> * _Nonnull messageList, ZIMMessage * _Nonnull nextMessage, ZIMError * _Nonnull errorInfo) {
            NSLog(@"search global local messages. next message: %@", nextMessage);
            
            // test next page
            ZIMMessageSearchConfig *config2 = [[ZIMMessageSearchConfig alloc] init];
            config2.count = 30;
            config2.nextMessage = nextMessage;
            config2.order = ZIMMessageOrderDescending;
            config2.startTime = config.startTime;
            config2.endTime = config.endTime;
            config2.messageTypes = config.messageTypes;
            config2.senderUserIDs = config.senderUserIDs;
            
            [[ZIM getInstance] searchGlobalLocalMessagesWithConfig:config2 callback:^(NSArray<ZIMMessage *> * _Nonnull messageList, ZIMMessage * _Nonnull nextMessage, ZIMError * _Nonnull errorInfo) {
                NSLog(@"search global local messages2. next message: %@", nextMessage);
            }];
        }];
    }
    
}

@end
