//
//  CreateGroupViewController.m
//  ZIMExampleDemo
//
//  Created by 武耀琳 on 2022/1/20.
//

#import "CreateGroupViewController.h"

#import "CreateGroupMemberListTableViewCell.h"
#import "ZGZIMManager.h"
#import "GroupViewController.h"
#import "ZGHUDHelper.h"
@interface CreateGroupViewController ()<UITableViewDelegate,UITableViewDataSource,groupMemberListDelegate>
@property (weak, nonatomic) IBOutlet UITextField *groupIDTextField;
@property (weak, nonatomic) IBOutlet UITextField *groupMemberTextField;
@property (weak, nonatomic) IBOutlet UITextField *groupNameTextField;


@property (weak, nonatomic) IBOutlet UIButton *addMemberButton;
@property (weak, nonatomic) IBOutlet UIButton *createGroupButton;

@property NSString *groupID;
@property NSString *groupMemberID;

@property (weak, nonatomic) IBOutlet UITableView *mumberTableView;

@property (weak, nonatomic) IBOutlet UIImageView *groupAvatarImageView;

@property int avatarCounter;
@property NSString *groupAvatar;


@end

@implementation CreateGroupViewController



- (void)viewDidLoad {
    [super viewDidLoad];
    self.groupID = @"";
    self.groupMemberID = @"";
//    [self.backButton setTitle:@"" forState:UIControlStateNormal];
    [self.addMemberButton setTitle:NSLocalizedString(@"add", nil) forState:UIControlStateNormal];
    [self.createGroupButton setTitle:NSLocalizedString(@"OK", nil) forState:UIControlStateNormal];
    _createGroupModel = [[CreateGroupModel alloc] init];
    [_createGroupModel addGroupMemberListDelegate:self];
    self.groupMemberTextField.clearButtonMode = UITextFieldViewModeWhileEditing;
}
- (IBAction)groupIDTextFieldDidChanged:(UITextField *)sender {
    self.groupID = sender.text;
}
- (IBAction)groupMemberTextFieldDidChanged:(UITextField *)sender {
    self.groupMemberID = sender.text;
}
- (IBAction)createGroupButtonClicked:(UIButton *)sender {
    ZIMGroupInfo *groupInfo = [[ZIMGroupInfo alloc] init];
    groupInfo.groupID = self.groupID;
    groupInfo.groupName = self.groupNameTextField.text;
    [ZGHUDHelper showNetworkLoading];
    groupInfo.groupAvatarUrl = self.groupAvatar?self.groupAvatar:@"";
    [[ZGZIMManager shared] createGroup:groupInfo userIDs:self.createGroupModel.createGroupMemberList callback:^(ZIMGroupFullInfo * _Nonnull groupInfo, NSArray<ZIMGroupMemberInfo *> * _Nonnull userList, NSArray<ZIMErrorUserInfo *> * _Nonnull errorUserList, ZIMError * _Nonnull errorInfo) {
        [ZGHUDHelper hideNetworkLoading];
        if(errorInfo.code == 0){
            if(errorUserList.count >0){
                [ZGHUDHelper showMessage:[NSString stringWithFormat:@"Failed to add some group members:%@",errorUserList]];
            }
            GroupViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"GroupViewController"];
            vc.toGroupID = groupInfo.baseInfo.groupID;
            [self.navigationController pushViewController:vc animated:YES];
            NSMutableArray *marr = [[NSMutableArray alloc]initWithArray:self.navigationController.viewControllers];
            for (UIViewController *vc in marr) {
                if (vc == self) {
                    [marr removeObject:vc];
                    break;          //break一定要加，不加有时候有bug
                }
            }
            self.navigationController.viewControllers = marr;
          //  [self.view.window setRootViewController:vc];
        }else{
            [ZGHUDHelper showMessage:[NSString stringWithFormat:@"failed！,error code:%d,error message:%@",(int)errorInfo.code,errorInfo.message]];
        }
    }];

}

- (IBAction)backButtonClicked:(UIButton *)sender {
    UITabBarController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"TabBarViewController"];
    CATransition *animation = [CATransition animation];
    animation.duration = 0.15f;
    animation.type = kCATransitionPush;
    animation.removedOnCompletion = YES;
    animation.subtype = kCATransitionFromLeft;
    [self.view.window.layer addAnimation:animation forKey:nil];
    [self.view.window setRootViewController:vc];
}


- (IBAction)updateGroupAvatarButtonClicked:(UIButton *)sender {
    if(_avatarCounter == 5){
        _avatarCounter = 1;
    }else{
        _avatarCounter = _avatarCounter + 1;
    }
    
    self.groupAvatar = [NSString stringWithFormat:@"https://storage.zego.im/zim/example/web/assets/%d.jpeg",_avatarCounter];
    [self.groupAvatarImageView setImage:[UIImage imageWithData:[NSData dataWithContentsOfURL:[NSURL URLWithString:self.groupAvatar]]]];
}


//MARK: - TableViewDelegate
- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    return self.createGroupModel.createGroupMemberList.count;
    
}


- (nonnull UITableViewCell *)tableView:(nonnull UITableView *)tableView cellForRowAtIndexPath:(nonnull NSIndexPath *)indexPath {
    CreateGroupMemberListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"CreateGroupMemberListTableViewCell"];
    [cell.userNameLabel setText:self.createGroupModel.createGroupMemberList[indexPath.row]];
    cell.vc = self;
    return cell;
}
- (IBAction)addMemberButtonClicked:(UIButton *)sender {
    if(self.groupMemberID == nil || [self.groupMemberID isEqual:@""]){
        return;
    }
    [self.createGroupModel addGroupMemberList:self.groupMemberID];
    self.groupMemberTextField.text = @"";
    self.groupMemberID = @"";
}
//MARK: - groupMemberListDelegate
-(void)groupMemberListupdate:(NSArray *)groupMmemberList{
    [self.mumberTableView reloadData];
}

@end
