//
//  GroupViewController.m
//  ZIMExampleDemo
//
//  Created by 武耀琳 on 2022/1/21.
//

#import "GroupViewController.h"
#import "ZGZIMManager.h"
#import "GroupReceiveMsgTableViewCell.h"
#import "GroupSendMsgTableViewCell.h"
#import "ConversationListDataSource.h"
#import "GroupSetViewController.h"
#import "ZGHUDHelper.h"
#import "KeyCenter.h"
#import "MJRefresh.h"
#import "GroupReceiveMsgImage.h"
#import <AVFoundation/AVFoundation.h>
#import <AVKit/AVKit.h>
#import "GroupReveiveMsgVideoTableViewCell.h"
#import "GroupSendMsgImageTableViewCell.h"
#import "GroupSendMsgFileTableViewCell.h"
#import "GroupReceiveMsgFileTableViewCell.h"
#import "GroupSendMsgVideoTableViewCell.h"
#import "GroupReveiveMsgVideoTableViewCell.h"
#import <Photos/Photos.h>
#import "SelectCallMembersViewController.h"
#import "RevokeMsgTableViewCell.h"

@interface GroupViewController ()<UITableViewDelegate,UITableViewDataSource,UITextViewDelegate,ZIMEventDelegate,UIImagePickerControllerDelegate,UINavigationControllerDelegate,UIDocumentPickerDelegate>
@property AVPlayer *myPlayer;

@property (weak, nonatomic) IBOutlet UITextView *sendMsgTextView;

@property (weak, nonatomic) IBOutlet UITableView *groupTableView;

@property (weak, nonatomic) IBOutlet UILabel *groupIDLabel;
@property (weak, nonatomic) IBOutlet UILabel *onlineMemberCountLabel;
@property unsigned int onlineMemberCount;

@property NSString *filePath;

@property NSString *imagePath;

@property (weak, nonatomic) IBOutlet UIImageView *videoImageView;
@property (weak, nonatomic) IBOutlet UILabel *videoMessageLabel;

@property (nonatomic,assign)NSInteger timeSecond;
//View
@property (weak, nonatomic) IBOutlet UIView *nowChatView;

@property UIView *chatSetView;

@property UIView *chatMainView;

@property (weak, nonatomic) IBOutlet UIButton *sendButton;

@property (weak, nonatomic) IBOutlet UIImageView *receiptImageView;

@end

@implementation GroupViewController

- (void)willMoveToParentViewController:(UIViewController*)parent{
    [super willMoveToParentViewController:parent];
    if(parent == nil){
        [[ZGZIMManager shared] removeZIMEventDelegate:self];
    }
}



- (void)viewDidLoad {
    [super viewDidLoad];
    self.navigationItem.title = self.toGroupID;
    [self.sendButton setTitle:NSLocalizedString(@"send",nil) forState:UIControlStateNormal];
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"set-2"] style:UIBarButtonItemStyleDone target:self action:@selector(setGroupButtonClicked)];
    self.sendMsgTextView.layer.borderWidth = 2;
    [self.groupIDLabel setText:self.toGroupID];
    [self queryHistoryMessage];
    [[ZGZIMManager shared] addZIMEventDelegate:self];
    [self clearConversationUnreadMessageCount];
    self.isclearUnreadMessage = true;
    self.groupTableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingBlock:^{
        [self queryHistoryMessage];
        [self.groupTableView.mj_header endRefreshing];
    }];
    [[ZGZIMManager shared] queryGroupMemberCountByGroupID:self.toGroupID callback:^(NSString * _Nonnull groupID, unsigned int count, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code == 0){
            [self.onlineMemberCountLabel setText:[NSString stringWithFormat:@"(%u)",count]];
            self.onlineMemberCount = count;
        }
    }];
    [[ZGZIMManager shared] sendConversationMessageReceiptRead:self.toGroupID conversationType:ZIMConversationTypeGroup callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {
        
    }];
    UILongPressGestureRecognizer *gesture = [[UILongPressGestureRecognizer alloc]  initWithTarget:self action:@selector(didLongPressTableViewCell:)];
    gesture.minimumPressDuration = 1.0;
    [self.groupTableView addGestureRecognizer:gesture];
}

-(void)clearConversationUnreadMessageCount{
    [[ZGZIMManager shared] clearConversationUnreadMessageCount:self.toGroupID type:2 callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {
    }];
}
-(void)queryHistoryMessage{
    ZIMMessageQueryConfig *config = [[ZIMMessageQueryConfig alloc] init];
    config.count = 20;
    config.reverse = true;
    bool firstQuery = false;
    if(self.historyMessageList.count>0){
        config.nextMessage = self.historyMessageList[0];
        firstQuery = true;
    }
    [[ZGZIMManager shared] queryHistoryMessage:self.toGroupID conversationType:2 config:config callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, NSArray<ZIMMessage *> * _Nonnull messageList, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code == 0){
            if(self.historyMessageList == nil){
                self.historyMessageList = (NSMutableArray *)messageList;
            }else{
                NSMutableArray *newHistoryMessageList = (NSMutableArray *)messageList;
                for (ZIMMessage *message in self.historyMessageList) {
                    [newHistoryMessageList addObject:message];
                }
                self.historyMessageList = newHistoryMessageList;
            }
            [self initMsgListTableView:self.historyMessageList firstQuery:firstQuery];
        }
    }];
}

-(void)deleteLocalHistoryMessage{
    
    self.historyMessageList = [[NSMutableArray<ZIMMessage *> alloc] init];
    
    [self.groupTableView reloadData];
}

-(void)initMsgListTableView:(NSMutableArray<ZIMMessage *> *)messageList
                 firstQuery:(BOOL)firstQuery{
    [self renewTableView:firstQuery==true?true:false];
}
//MARK: -
- (IBAction)backButtonClicked:(UIButton *)sender {
    UITabBarController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"TabBarViewController"];
    vc.selectedIndex = self.tabBarIndex;
    CATransition *animation = [CATransition animation];
    animation.duration = 0.15f;
    animation.type = kCATransitionPush;
    animation.removedOnCompletion = YES;
    animation.subtype = kCATransitionFromLeft;
    [self.view.window.layer addAnimation:animation forKey:nil];
    [self.view.window setRootViewController:vc];
    [[ZGZIMManager shared] removeZIMEventDelegate:self];
}

- (void)setGroupButtonClicked{
    self.isclearUnreadMessage = false;
    GroupSetViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"GroupSetViewController"];
    vc.myGroupID = self.toGroupID;
    vc.masterVc = self;
    [self.navigationController pushViewController:vc animated:YES];
  //  [self.view.window setRootViewController:vc];    
}

- (IBAction)sendMessageButtonClicked:(UIButton *)sender {
    ZIMTextMessage *txtMsg = [[ZIMTextMessage alloc] init];
    txtMsg.message = self.sendMsgTextView.text;
    ZIMPushConfig *pushConfig = [[ZIMPushConfig alloc] init];
    pushConfig.title = self.toGroupID;
    pushConfig.content = self.sendMsgTextView.text;
    pushConfig.resourcesID = KeyCenter.resourceID;
    ZIMMessageSendConfig *config = [[ZIMMessageSendConfig alloc] init];
    config.pushConfig = pushConfig;
    config.hasReceipt = !self.receiptImageView.hidden;
    config.priority = 1;
    [[ZGZIMManager shared] sendMessage:txtMsg toConversationID:self.toGroupID conversationType:ZIMConversationTypeGroup config:config notification:nil callback:^(ZIMMessage * _Nonnull message, ZIMError * _Nonnull errorInfo) {
        [self renewTableView:false];
    }];
    [self.historyMessageList addObject:txtMsg];
    [self renewTableView:false];
    [self.sendMsgTextView setText:@""];
}

- (void)receiveGroupMessage:(NSArray<ZIMMessage *> *)messageList
                fromGroupID:(NSString *)fromGroupID{
    if(![fromGroupID isEqual:self.toGroupID]) return;
    NSArray *reverseMsgList = [[messageList reverseObjectEnumerator] allObjects];
    for (ZIMMessage *msg in reverseMsgList) {
        [self.historyMessageList addObject:msg];
        if(msg.receiptStatus == ZIMMessageReceiptStatusProcessing){
            [[ZGZIMManager shared] sendMessageReceiptsRead:@[msg] conversationID:self.toGroupID conversationType:ZIMConversationTypeGroup callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, NSArray<NSNumber *> * _Nonnull errorMessageIDs, ZIMError * _Nonnull errorInfo) {
                [self.historyMessageList[[self.historyMessageList indexOfObject:msg]] setValue:[NSNumber numberWithUnsignedInteger:ZIMMessageReceiptStatusDone] forKey:@"receiptStatus"];
                [self.groupTableView reloadData];
            }];
        }
    }
    [self renewTableView:false];
    if(self.isclearUnreadMessage == true){
        [[ZGZIMManager shared] clearConversationUnreadMessageCount:self.toGroupID type:2 callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {}];
    }
}
-(void)renewTableView:(BOOL)isTop{
    [self.groupTableView reloadData];
    dispatch_async(dispatch_get_main_queue(), ^{
        if(isTop == false){
            if(self.historyMessageList.count>0){
                [self.groupTableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:self.historyMessageList.count -1 inSection:0]  atScrollPosition:UITableViewScrollPositionBottom animated:YES];
            }
        }
    });
}

- (IBAction)addSetButtonClicked:(UIButton *)sender {
    _chatSetView = [[[UINib nibWithNibName:@"GroupChatSetView" bundle:[NSBundle mainBundle]]
                     instantiateWithOwner:self options:nil] lastObject];
    [_chatSetView setTag:1000];
    CGRect rect = _chatSetView.frame;
    rect.size.width = [UIScreen mainScreen].bounds.size.width;
    [_chatSetView setFrame:rect];
    _chatMainView = [self.nowChatView.subviews firstObject];
    [self.nowChatView addSubview:_chatSetView];
}


- (IBAction)photoButtonClicked:(UIButton *)sender {
    if([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypeSavedPhotosAlbum]){

        UIImagePickerController *imagePickerCtrl = [[UIImagePickerController alloc] init];
        imagePickerCtrl.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
        imagePickerCtrl.delegate = (id)self;
        
        //指定媒体类型,图片(@"public.image"),视频(@"public.movie")
        imagePickerCtrl.mediaTypes = @[@"public.image",@"public.movie"];
        [self presentViewController:imagePickerCtrl animated:YES completion:NULL];
    }
}
- (IBAction)folderButtonClicked:(UIButton *)sender {
    NSArray *types = @[@"public.content",@"public.text"];
    UIDocumentPickerViewController *picker = [[UIDocumentPickerViewController alloc] initWithDocumentTypes:types inMode:UIDocumentPickerModeOpen];
    picker.delegate = (id)self;
    [self presentViewController:picker animated:YES completion:nil];
}

- (IBAction)callInviteButtonClicked:(UIButton *)sender {
    

    ZIMGroupMemberQueryConfig *queryConfig = [[ZIMGroupMemberQueryConfig alloc] init];
    queryConfig.count = 20;
    [[ZGZIMManager shared] queryGroupMemberListByGroupID:self.toGroupID config:queryConfig callback:^(NSString * _Nonnull groupID, NSArray<ZIMGroupMemberInfo *> * _Nonnull userList, unsigned int nextFlag, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code == 0){
            SelectCallMembersViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"SelectCallMembersViewController"];
            
            [self.navigationController pushViewController:vc animated:NO];
            [vc addList:userList];
        }
    }];

}

- (IBAction)receiptButtonClicked:(UIButton *)sender {
    self.receiptImageView.hidden = !self.receiptImageView.hidden;
    [_chatSetView removeFromSuperview];
}

-(void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
    if([touches anyObject].view.tag != 1000){
        NSLog(@"点击到外部");
        [_chatSetView removeFromSuperview];
        [self.nowChatView addSubview:_chatMainView];
    }else{
        NSLog(@"点击到内部");
    }
    
}

-(void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<NSString *,id> *)info
{
    
    //拿到图片会就销毁之前的控制器
    [picker dismissViewControllerAnimated:YES completion:^{
        if([[info objectForKey:UIImagePickerControllerMediaType] isEqual:@"public.movie"]){
            
            
            NSURL *sourceURL = [info objectForKey:UIImagePickerControllerMediaURL];
            NSString  *videoPath;
            
            if(sourceURL == nil){
                sourceURL = [info objectForKey:@"UIImagePickerControllerReferenceURL"];
                videoPath = sourceURL.baseURL.path;
            }else{
                videoPath =  info[UIImagePickerControllerMediaURL];
            }
            
            [[NSFileManager defaultManager] removeItemAtPath:self.filePath error:nil];
            [[NSFileManager defaultManager] removeItemAtPath:self.imagePath error:nil];
            NSDateFormatter *formater = [[NSDateFormatter alloc] init];
            [formater setDateFormat:@"yy-MM-dd-HH:mm:ss"];
            
            self.filePath = [NSHomeDirectory() stringByAppendingFormat:@"/Documents/%@", [[formater stringFromDate:[NSDate date]] stringByAppendingString:@".mp4"]];


            NSFileManager *fileManager = [NSFileManager defaultManager];

            NSError *error;
            [fileManager copyItemAtPath:videoPath toPath:self.filePath error:&error];
            if (error)
            {

                NSLog(@"文件保存到缓存失败");
            }

            
            [sourceURL startAccessingSecurityScopedResource];
            ZIMVideoMessage *videoMsg = [[ZIMVideoMessage alloc] initWithFileLocalPath:self.filePath videoDuration:10];
            [videoMsg setValue:[NSNumber numberWithInt:3] forKey:@"sentStatus"];
            [self.historyMessageList addObject:videoMsg];
            [self renewTableView:false];

        }else if ([[info objectForKey:UIImagePickerControllerMediaType] isEqual:@"public.image"]){
    //        info中就是包含你在相册里面选择的图片(info字典里面还有图片的url-有需要的可以将info打印出来取自己要的数据)
            NSURL *url = [info objectForKey:@"UIImagePickerControllerImageURL"];
            
            NSString *urlStr = url.path;
            ZIMImageMessage *message = [[ZIMImageMessage alloc] initWithFileLocalPath:urlStr];
            
            
            [message setValue:[NSNumber numberWithInt:3] forKey:@"sentStatus"];
            [self.historyMessageList addObject:message];
            [self renewTableView:false];
        }
    }];
    
    
}

- (void)documentPicker:(UIDocumentPickerViewController *)controller didPickDocumentsAtURLs:(NSArray<NSURL *> *)urls {
    // 获取授权
    [urls.firstObject startAccessingSecurityScopedResource];
    
    ZIMFileMessage *fileMsg = [[ZIMFileMessage alloc] initWithFileLocalPath:[urls firstObject].path];
    ZIMMessageSendConfig *sendConfig = [[ZIMMessageSendConfig alloc] init];
    sendConfig.priority = 1;
    [fileMsg setValue:[NSNumber numberWithInt:3] forKey:@"sentStatus"];
    [self.historyMessageList addObject:fileMsg];
    [self renewTableView:false];
    
}


-(void)playSelectVideo:(NSURL *)url{
    AVPlayerViewController *playerVc = [[AVPlayerViewController alloc] init];
    AVPlayer *avPlayer= [AVPlayer playerWithURL:url];
    playerVc.player = avPlayer;
    [self presentViewController:playerVc animated:YES completion:nil];
    [playerVc.player play];
}


//MARK: - TableViewDelegate

- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {

    return _historyMessageList.count;
}
- (nonnull UITableViewCell *)tableView:(nonnull UITableView *)tableView cellForRowAtIndexPath:(nonnull NSIndexPath *)indexPath {
    NSLog(@"[510 log]%ld",(long)indexPath.row);
    ZIMMessageSendConfig *sendConfig = [[ZIMMessageSendConfig alloc] init];
    sendConfig.hasReceipt = !self.receiptImageView.hidden;
    ZIMMessage *msg = _historyMessageList[indexPath.row];
    void(^readayToShowCallback)(MessageCellEvent event,ZIMMessage * msg);
    MessageTableViewCell *cell;
    readayToShowCallback = ^(MessageCellEvent event,ZIMMessage * msg){
        switch (event) {
            case Revoke:{
                [self.historyMessageList replaceObjectAtIndex:indexPath.row withObject:msg];
                break;
            }
            case Delete:{
                [self.historyMessageList removeObject:msg];
                break;
            }
            case Update:{
                [self.historyMessageList replaceObjectAtIndex:indexPath.row withObject:msg];
            }
            default:
                break;
        }
        [tableView reloadData];
    };
    if(msg.direction == ZIMMessageDirectionSend){
        if(msg.type == ZIMMessageTypeText){
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupSendMsgTextTableViewCell"];
        }
        else if (msg.type == ZIMMessageTypeImage){
            
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupSendMsgImageTableViewCell"];
            ((GroupSendMsgImageTableViewCell *)cell).masterVC = (id)self;
            ((GroupSendMsgImageTableViewCell *)cell).sendConfig = sendConfig;
        }
        else if (msg.type == ZIMMessageTypeVideo){
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupSendMsgVideoTableViewCell"];
            ((GroupSendMsgVideoTableViewCell *)cell).masterVC = (id)self;
            ((GroupSendMsgVideoTableViewCell *)cell).sendConfig = sendConfig;
        }
        else if (msg.type == ZIMMessageTypeFile){
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupSendMsgFileTableViewCell"];
            
            ((GroupSendMsgFileTableViewCell *)cell).masterVC = (id)self;
            ((GroupSendMsgFileTableViewCell *)cell).sendConfig = sendConfig;
        }
        else if (msg.type == ZIMMessageTypeCommand){
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupSendMsgTextTableViewCell"];
            [((GroupSendMsgTableViewCell *)cell).UserNameLabel setText:[ZGZIMManager shared].myUserID];
            [((GroupSendMsgTableViewCell *)cell).MsgContentLabel setText:@"[cmd message]"];
        }
        else if(msg.type == ZIMMessageTypeRevoke){
            cell = [tableView dequeueReusableCellWithIdentifier:@"RevokeMsgTableCell"];
        }else{
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupSendMsgTextTableViewCell"];
        }
    }
    else if(msg.direction == ZIMMessageDirectionReceive){
        if(msg.type == ZIMMessageTypeText){
            
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupReceiveMsgTableViewCell"];
            [((GroupReceiveMsgTableViewCell *)cell).UserNameLabel setText:msg.senderUserID];
            [((GroupReceiveMsgTableViewCell *)cell).MsgContentLabel setText:((ZIMTextMessage *)msg).message];
            
        }
        else if (msg.type == ZIMMessageTypeImage){
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupReceiveMsgImage"];
        }
        else if (msg.type == ZIMMessageTypeVideo){
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupReveiveMsgVideoTableViewCell"];
            ((GroupReveiveMsgVideoTableViewCell *)cell).mastervc = (id)self;
        }
        else if (msg.type == ZIMMessageTypeAudio){
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupReceiveMsgFileTableViewCell"];
            [((GroupReceiveMsgFileTableViewCell *)cell).titleLabel setText:msg.senderUserID];
        }
        else if (msg.type == ZIMMessageTypeFile){
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupReceiveMsgFileTableViewCell"];
            [((GroupReceiveMsgFileTableViewCell *)cell).titleLabel setText:msg.senderUserID];
        }
        else if (msg.type == ZIMMessageTypeCommand){
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupReceiveMsgTableViewCell"];
            [((GroupReceiveMsgTableViewCell *)cell).UserNameLabel setText:msg.senderUserID];
            [((GroupReceiveMsgTableViewCell *)cell).MsgContentLabel setText:@"[receicve a cmd message]"];
        }
        else if(msg.type == ZIMMessageTypeRevoke){
            cell = [tableView dequeueReusableCellWithIdentifier:@"RevokeMsgTableCell"];
        }else{
            cell = [tableView dequeueReusableCellWithIdentifier:@"GroupReceiveMsgTableViewCell"];
            [((GroupReceiveMsgTableViewCell *)cell).UserNameLabel setText:msg.senderUserID];
            [((GroupReceiveMsgTableViewCell *)cell).MsgContentLabel setText:@"[未知消息类型]"];
        }
    }
    [cell readyToshow:msg callback:readayToShowCallback];
    return cell;
}

- (void)didLongPressTableViewCell:(UILongPressGestureRecognizer *)gesture
{
    if (gesture.state == UIGestureRecognizerStateBegan) {
        //获取到点击的位置
        CGPoint point = [gesture locationInView:self.groupTableView];
        
        NSIndexPath *indexPath = [self.groupTableView indexPathForRowAtPoint:point];
        UITableViewCell *cell = [self.groupTableView cellForRowAtIndexPath:indexPath];
        [cell becomeFirstResponder];
        NSValue *pointValue =  [NSValue valueWithCGPoint:point];
        [cell performSelector:@selector(showMessageMenu:) withObject:pointValue];
        if(indexPath == nil) return;
        
        //do something...
    }
}

- (void)groupMemberStateChanged:(ZIMGroupMemberState)state
                      event:(ZIMGroupMemberEvent)event
                   userList:(NSArray<ZIMGroupMemberInfo *> *)userList
               operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
                        groupID:(NSString *)groupID{
    if(state == ZIMGroupMemberStateEnter){
        self.onlineMemberCount = self.onlineMemberCount + (unsigned int)userList.count;
        [self.onlineMemberCountLabel setText:[NSString stringWithFormat:@"(%u)",self.onlineMemberCount]];
    }else if (state == ZIMGroupMemberStateQuit){
        self.onlineMemberCount = self.onlineMemberCount - (unsigned int)userList.count;
        [self.onlineMemberCountLabel setText:[NSString stringWithFormat:@"(%u)",self.onlineMemberCount]];
    }
}

- (void)messageRevokeReceived:(NSArray<ZIMRevokeMessage *> *)messageList{
    for (ZIMRevokeMessage *revokeMessage in messageList) {
        if(revokeMessage.conversationType != ZIMConversationTypeGroup){
            continue;
        }
        for (ZIMMessage *message in self.historyMessageList) {
            if(message.messageID == revokeMessage.messageID){
                [self.historyMessageList replaceObjectAtIndex:[self.historyMessageList indexOfObject:message] withObject:revokeMessage];
                break;
            }
        }
    }
    [self.groupTableView reloadData];
}

- (void)zim:(ZIM *)zim conversationMessageReceiptChanged:(NSArray<ZIMMessageReceiptInfo *> *)infos{
    for (ZIMMessageReceiptInfo *receiptInfo in infos) {
        if(![receiptInfo.conversationID isEqual: self.toGroupID]){
            return;
        }
        for (int index = 0; index< self.historyMessageList.count; index ++) {
            if(self.historyMessageList[index].messageID == receiptInfo.messageID){
                [self.historyMessageList[index] setValue:[NSNumber numberWithUnsignedInteger:receiptInfo.status] forKey:@"receiptStatus"];
                break;
            }
        }
        [self renewTableView:false];
    }
}

- (void)zim:(ZIM *)zim messageReceiptChanged:(NSArray<ZIMMessageReceiptInfo *> *)infos{
    for (ZIMMessageReceiptInfo *receiptInfo in infos) {
        if(![receiptInfo.conversationID isEqual: self.toGroupID]){
            return;
        }
        for (int index = 0; index< self.historyMessageList.count; index ++) {
            if(self.historyMessageList[index].messageID == receiptInfo.messageID){
                [self.historyMessageList[index] setValue:[NSNumber numberWithUnsignedInteger:receiptInfo.status] forKey:@"receiptStatus"];
                break;
            }
        }
        [self renewTableView:false];
    }
}

@end
