//
//  GroupSetViewController.m
//  ZIMExampleDemo
//
//  Created by 武耀琳 on 2022/2/17.
//

#import "GroupSetViewController.h"
#import "ZGZIMManager.h"
#import "ZGHUDHelper.h"
#import "ConversationListDataSource.h"
#import "GroupSetNumberTableViewCell.h"
#import "GroupMemberManagerTableViewCell.h"
#import "ChangeGroupNameViewController.h"
#import "ChangeGroupBoardViewController.h"
#import "ChangeGroupMarkViewController.h"
#import "transferGroupOwnerViewController.h"
#import "RemoveGroupMemberViewController.h"
#import "AddGroupMemberViewController.h"
@interface GroupSetViewController ()<UITableViewDelegate,UITableViewDataSource,ZIMEventDelegate>
@property (weak, nonatomic) IBOutlet UILabel *groupNameLabel;

@property (weak, nonatomic) IBOutlet UITableView *groupMemberListTableView;
@property ZIMGroupInfo *myGroupInfo;
@property (weak, nonatomic) IBOutlet UITextField *groupNameTextField;
@property (weak, nonatomic) IBOutlet UITextView *groupBoardTextView;
@property (weak, nonatomic) IBOutlet UITextView *groupReMarkTextView;

@property (weak, nonatomic) IBOutlet UIButton *groupNameButton;

@property (weak, nonatomic) IBOutlet UIButton *groupNoticeButton;
@property (weak, nonatomic) IBOutlet UIButton *groupAttributesButton;
@property (weak, nonatomic) IBOutlet UILabel *transferLabel;
@property (weak, nonatomic) IBOutlet UIButton *transferButton;


@property (weak, nonatomic) IBOutlet UIButton *clearCharLogsButton;

@property (weak, nonatomic) IBOutlet UIButton *dismissGroupButton;
@property (weak, nonatomic) IBOutlet UIButton *deleteGroupButton;

@property NSMutableArray<ZIMGroupMemberInfo *> * _Nonnull userList;
@property unsigned int nextFlag;



@end

@implementation GroupSetViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self.clearCharLogsButton setTitle:NSLocalizedString(@"clear chat logs",nil)  forState:UIControlStateNormal];
    [self.deleteGroupButton setTitle:NSLocalizedString(@"delete and quit group chat",nil)  forState:UIControlStateNormal];
    [self.dismissGroupButton setTitle:NSLocalizedString(@"dismiss and quit group chat",nil)  forState:UIControlStateNormal];
    self.navigationItem.title = NSLocalizedString(@"Group Setting", nil);
    [[ZGZIMManager shared] addZIMEventDelegate:self];
    self.groupNameTextField.layer.borderWidth = 1;
    self.groupBoardTextView.layer.borderWidth = 1;
    self.groupReMarkTextView.layer.borderWidth = 1;
    [self queryGroupMemberList];
    
    
    
    [self hiddenGroupOwnerItem];
    [[ZGZIMManager shared] queryGroupInfo:self.myGroupID callback:^(ZIMGroupFullInfo * _Nonnull groupInfo, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code == 0){
            self.myGroupInfo = groupInfo.baseInfo;
            if([groupInfo.baseInfo.groupName isEqual:@""] ||groupInfo.baseInfo.groupName == nil){
                self.groupNameTextField.text =NSLocalizedString(@"The group chat name is not set",nil);
                UIColor *color = [UIColor grayColor];
                [self.groupNameTextField setTextColor:color];
            }else{
                self.groupNameTextField.text = groupInfo.baseInfo.groupName;
            }
            if ([groupInfo.groupNotice isEqual:@""] || groupInfo.groupNotice == nil){
                self.groupBoardTextView.text = NSLocalizedString(@"Group notice is not set.",nil);
                UIColor *color = [UIColor grayColor];
                [self.groupBoardTextView setTextColor:color];
            }else{
                self.groupBoardTextView.text = groupInfo.groupNotice;
            }
        }
    }];
    NSArray *groupAttributesKeys = @[@"GroupMark"];
    [[ZGZIMManager shared] queryGroupAttributes:groupAttributesKeys groupID:self.myGroupID callback:^(NSString * _Nonnull groupID, NSDictionary<NSString *,NSString *> * _Nonnull groupAttributes, ZIMError * _Nonnull errorInfo) {
//        NSString *groupMark = [groupAttributes objectForKey:@"GroupMark"];
//        if(errorInfo.code ==0){
//            if([groupMark isEqual:@""]||groupMark == nil){
//                self.groupReMarkTextView.text = NSLocalizedString(@"No group remarks are available",nil);
//                UIColor *color = [UIColor grayColor];
//                [self.groupReMarkTextView setTextColor:color];
//            }else{
//                self.groupReMarkTextView.text = groupMark;
//                UIColor *color = [UIColor blackColor];
//                [self.groupReMarkTextView setTextColor:color];
//            }
//        }
    }];
}

-(void)queryGroupMemberList{
    ZIMGroupMemberQueryConfig *config = [[ZIMGroupMemberQueryConfig alloc] init];
    config.count = 500;
    [[ZGZIMManager shared] queryGroupMemberListByGroupID:self.myGroupID config:config callback:^(NSString * _Nonnull groupID, NSArray<ZIMGroupMemberInfo *> * _Nonnull userList, unsigned int nextFlag, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code == 6000521){
            [ZGHUDHelper showMessage:[NSString stringWithFormat:NSLocalizedString(@"You are no longer in the group",nil)]];
            CATransition *animation = [CATransition animation];
            animation.duration = 0.15f;
            animation.type = kCATransitionPush;
            animation.removedOnCompletion = YES;
            animation.subtype = kCATransitionFromLeft;
            [self.view.window.layer addAnimation:animation forKey:nil];
            [self.view.window setRootViewController:self.masterVc];
            return;
        }
        if(errorInfo.code != 0){
            [ZGHUDHelper showMessage:[NSString stringWithFormat:@"failed,error code:%u,error message:%@",(int)errorInfo.code,errorInfo.message]];
            CATransition *animation = [CATransition animation];
            animation.duration = 0.15f;
            animation.type = kCATransitionPush;
            animation.removedOnCompletion = YES;
            animation.subtype = kCATransitionFromLeft;
            [self.view.window.layer addAnimation:animation forKey:nil];
            [self.view.window setRootViewController:self.masterVc];
            return;
        }
        self.userList = (NSMutableArray *)userList;
        [self findGroupOwner];
        [self.groupMemberListTableView reloadData];
        self.groupMemberListTableView.hidden = false;
    }];

}

-(void)findGroupOwner{
    for (ZIMGroupMemberInfo *memberInfo in self.userList) {
        if([memberInfo.userID isEqual:[ZGZIMManager shared].myUserID]){
            if(memberInfo.memberRole == 1){
                self.isGroupOwner = true;
                [self unhiddenGroupOwnerItem];
            }
            else{
                self.isGroupOwner = false;
            }
        }
    }
}


-(void)hiddenGroupOwnerItem{
    self.groupNoticeButton.hidden = true;
    self.transferLabel.hidden = true;
    self.transferButton.hidden = true;
    self.dismissGroupButton.hidden = true;
}

-(void)unhiddenGroupOwnerItem{
    self.groupNoticeButton.hidden = false;
    self.transferLabel.hidden = false;
    self.transferButton.hidden = false;
    self.dismissGroupButton.hidden = false;
}
//MARK: - buttonClicked
- (IBAction)backButtonClicked:(UIButton *)sender {
    self.masterVc.isclearUnreadMessage = true;
    [self.masterVc clearConversationUnreadMessageCount];
    [self.navigationController popViewControllerAnimated:YES];
   // [self.view.window setRootViewController:self.masterVc];
    
}

- (IBAction)addGroupMemberButtonClicked:(UIButton *)sender {
    AddGroupMemberViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"AddGroupMemberViewController"];
    vc.myGroupID = self.myGroupID;
    vc.masterVC = self;
    [self.navigationController pushViewController:vc animated:YES];
}

- (IBAction)removeGroupMemberButtonClicked:(UIButton *)sender {
    RemoveGroupMemberViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"RemoveGroupMemberViewController"];
    vc.myGroupID = self.myGroupID;
    vc.masterVC = self;
    [self.navigationController pushViewController:vc animated:YES];
}

- (IBAction)changeGroupNameButtonClicked:(UIButton *)sender {
    ChangeGroupNameViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"ChangeGroupNameViewController"];
    vc.myGroupID = self.myGroupID;
    vc.masterVC = self;
    [self.navigationController pushViewController:vc animated:YES];
    
}
- (IBAction)changeGroupBoardButtonClicked:(UIButton *)sender {
    ChangeGroupBoardViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"ChangeGroupBoardViewController"];
    vc.myGroupID = self.myGroupID;
    vc.masterVC = self;
    [self.navigationController pushViewController:vc animated:YES];
}

- (IBAction)changeGroupMarkButtonClicked:(UIButton *)sender {
    ChangeGroupMarkViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"ChangeGroupMarkViewController"];
    vc.masterVC = self;
    vc.myGroupID = self.myGroupID;
    [self.navigationController pushViewController:vc animated:YES];
}
- (IBAction)transferGroupOwnerButtonClicked:(UIButton *)sender {
    TransferGroupOwnerViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"TransferGroupOwnerViewController"];
    vc.myGroupID = self.myGroupID;
    vc.masterVC = self;
    [self.navigationController pushViewController:vc animated:YES];
}

- (IBAction)clearChattingRecordsButtonClicked:(UIButton *)sender {
    ZIMMessageDeleteConfig *config = [[ZIMMessageDeleteConfig alloc] init];
    config.isAlsoDeleteServerMessage = false;
    [[ZGZIMManager shared] deleteMessages:[self.masterVc historyMessageList] conversationID:self.myGroupID conversationType:2 config:config callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code == 0){
            [self.masterVc deleteLocalHistoryMessage];
        }
    }];
}
- (IBAction)quitGroupBUttonClicked:(UIButton *)sender {
    [[ZGZIMManager shared] leaveGroup:self.myGroupID callback:^(NSString * _Nonnull groupID, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code == 0){
            ZIMConversationDeleteConfig *config = [[ZIMConversationDeleteConfig alloc] init];
            config.isAlsoDeleteServerConversation = true;
            [[ZGZIMManager shared] deleteConversation:self.myGroupID type:2 config:config callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {}];

            [[ConversationListDataSource shared] deleteConversationByConversationID:self.myGroupID];
            NSInteger viewControllersCount = self.navigationController.viewControllers.count;
            [self.navigationController popToViewController:self.navigationController.viewControllers[viewControllersCount -1 -2] animated:YES];
        }
    }];

    
}
- (IBAction)dismissGroupButtonClicked:(UIButton *)sender {
    [[ZGZIMManager shared] dismissGroup:self.myGroupID callback:^(NSString * _Nonnull groupID, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code == 0){
            ZIMConversationDeleteConfig *config = [[ZIMConversationDeleteConfig alloc] init];
            config.isAlsoDeleteServerConversation = true;
            [[ZGZIMManager shared] deleteConversation:self.myGroupID type:2 config:config callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {}];
            [[ConversationListDataSource shared] deleteConversationByConversationID:self.myGroupID];
            NSInteger viewControllersCount = self.navigationController.viewControllers.count;
            [self.navigationController popToViewController:self.navigationController.viewControllers[viewControllersCount -1 -2] animated:YES];

        }
    }];
}


//MARK: - TableViewDelegate
- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    int remainder = self.userList.count%4;
    if(remainder == 0){
        NSLog(@"count510:%lu",self.userList.count);
        NSLog(@"totalRow:%lu",self.userList.count/4);
        return self.userList.count/4 + 1;
        
    }else{
        NSLog(@"totalRow:%lu",self.userList.count/4+1);
        return self.userList.count/4 + 2;
    }
}
-(void)addIDLabelValue:(UILabel *)idLabel
         groupMemberInfo:(ZIMGroupMemberInfo *)memberInfo{
//    if(memberInfo.memberNickname != nil && ![memberInfo.memberNickname isEqual:@""]){
//        nameLabel.text = memberInfo.memberNickname;
//    }else if (memberInfo.userName != nil && ![memberInfo.userName isEqual:@""]){
//        nameLabel.text = memberInfo.userName;
//    }else{
//        nameLabel.text = memberInfo.userID;
//    }
    idLabel.text = memberInfo.userID;
}

-(void)addNameLabelValue:(UILabel *)nameLabel
         groupMemberInfo:(ZIMGroupMemberInfo *)memberInfo{
    nameLabel.text = memberInfo.userName;
}

-(void)addMemberAvatar:(UIImageView *)imageView
       groupMemberInfo:(ZIMGroupMemberInfo *)memberInfo{
    if(![memberInfo.memberAvatarUrl isEqual:@""]){
        [imageView setImage:[UIImage imageWithData:[NSData dataWithContentsOfURL:[NSURL URLWithString:memberInfo.memberAvatarUrl]]]];
    }
    
}
        

-(void)reloadMyTableView{
    [self.groupMemberListTableView reloadData];
}

- (nonnull UITableViewCell *)tableView:(nonnull UITableView *)tableView cellForRowAtIndexPath:(nonnull NSIndexPath *)indexPath {
    if(indexPath.row < self.userList.count / 4){
        NSLog(@"1IFRow%ld",(long)indexPath.row);
        GroupSetNumberTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"GroupSetNumberTableViewCell"];
        cell.groupID =self.myGroupID;
       // NSRange range = {indexPath.row*4+0,indexPath.row*4+3};
        //NSArray *groupMembers = [self.userList subarrayWithRange:range];
      //  [cell readyToShow:groupMembers];
        ZIMGroupMemberInfo *memberInfo0 = self.userList[indexPath.row*4 + 0];
        [self addIDLabelValue:cell.user1NameLabel groupMemberInfo:memberInfo0];
        [self addNameLabelValue:cell.user1trueNameLabel groupMemberInfo:memberInfo0];
        [self addMemberAvatar:cell.user1ImageView groupMemberInfo:memberInfo0];
        
        ZIMGroupMemberInfo *memberInfo1 = self.userList[indexPath.row*4 + 1];
        [self addIDLabelValue:cell.user2NameLabel groupMemberInfo:memberInfo1];
        [self addNameLabelValue:cell.user2TrueNameLabel groupMemberInfo:memberInfo1];
        [self addMemberAvatar:cell.user2ImageView groupMemberInfo:memberInfo1];
        ZIMGroupMemberInfo *memberInfo2 = self.userList[indexPath.row*4 + 2];
        [self addIDLabelValue:cell.user3NameLabel groupMemberInfo:memberInfo2];
        [self addNameLabelValue:cell.user3TrueNameLabel groupMemberInfo:memberInfo2];
        [self addMemberAvatar:cell.user3ImageView groupMemberInfo:memberInfo2];
        ZIMGroupMemberInfo *memberInfo3 = self.userList[indexPath.row*4 + 3];
        [self addIDLabelValue:cell.user4NameLabel groupMemberInfo:memberInfo3];
        [self addNameLabelValue:cell.user4TrueNameLabel groupMemberInfo:memberInfo3];
        [self addMemberAvatar:cell.user4ImageView groupMemberInfo:memberInfo3];
        [cell recoverHidden];
        return cell;
    }else if (indexPath.row == self.userList.count / 4){
        
        int remainder = self.userList.count%4;
        if(remainder == 0){
            NSLog(@"2IFRow%ld",(long)indexPath.row);
            GroupMemberManagerTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"GroupMemberManagerTableViewCell"];
            if(self.isGroupOwner == true){
                [cell reconverMySelf];
            }else{
                [cell hiddenMySelf];
            }
            return cell;
        }
        
        GroupSetNumberTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"GroupSetNumberTableViewCell"];
        cell.groupID = _myGroupID;
        [cell recoverHidden];
        if(remainder >= 1){
            ZIMGroupMemberInfo *memberInfo0 = self.userList[indexPath.row*4 + 0];
            [self addIDLabelValue:cell.user1NameLabel groupMemberInfo:memberInfo0];
            [self addNameLabelValue:cell.user1trueNameLabel groupMemberInfo:memberInfo0];
            [self addMemberAvatar:cell.user1ImageView groupMemberInfo:memberInfo0];
        }
        if(remainder >= 2){
            ZIMGroupMemberInfo *memberInfo1 = self.userList[indexPath.row*4 + 1];
            [self addIDLabelValue:cell.user2NameLabel groupMemberInfo:memberInfo1];
            [self addNameLabelValue:cell.user2TrueNameLabel groupMemberInfo:memberInfo1];
            [self addMemberAvatar:cell.user2ImageView groupMemberInfo:memberInfo1];
        }
        if(remainder >= 3){
            ZIMGroupMemberInfo *memberInfo2 = self.userList[indexPath.row*4 + 2];
            [self addIDLabelValue:cell.user3NameLabel groupMemberInfo:memberInfo2];
            [self addNameLabelValue:cell.user3TrueNameLabel groupMemberInfo:memberInfo2];
            [self addMemberAvatar:cell.user3ImageView groupMemberInfo:memberInfo2];
        }
        [cell hiddenSomeMember:4-remainder];
        return cell;
        
    }
    else if(indexPath.row == self.userList.count / 4 + 1){
        GroupMemberManagerTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"GroupMemberManagerTableViewCell"];
        if(self.isGroupOwner == true){
            [cell reconverMySelf];
        }else{
            [cell hiddenMySelf];
        }
        return cell;
    }
    else{
        return 0;
    }
}
//MARK: -ZIMEventDelegate
- (void)groupStateChanged:(ZIMGroupState)state
                event:(ZIMGroupEvent)event
         operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
                groupInfo:(ZIMGroupFullInfo *)groupInfo{
    
}

- (void)groupNameUpdated:(NSString *)groupName
        operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
                 groupID:(NSString *)groupID{
    if(groupID != self.myGroupID){
        return;
    }
    self.groupNameTextField.text =groupName;
}

- (void)groupNoticeUpdated:(NSString *)groupNotice
          operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
                   groupID:(NSString *)groupID{
    if(groupID != self.myGroupID){
        return;
    }
    self.groupBoardTextView.text = groupNotice;
    UIColor *color = [UIColor blackColor];
    [self.groupBoardTextView setTextColor:color];
}

- (void)groupAttributesUpdated:(NSArray<ZIMGroupAttributesUpdateInfo *> *)updateInfo
              operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
                       groupID:(NSString *)groupID{
    if(![groupID isEqual: self.myGroupID]){
        return;
    }
    for (ZIMGroupAttributesUpdateInfo *info in updateInfo) {
        if(info.action == ZIMGroupAttributesUpdateActionSet){
            NSString *groupMark = [info.groupAttributes objectForKey:@"GroupMark"];
            self.groupReMarkTextView.text = groupMark;
        }
        else if (info.action == ZIMGroupAttributesUpdateActionDelete){
            
        }
    }
}


- (void)groupMemberStateChanged:(ZIMGroupMemberState)state
                      event:(ZIMGroupMemberEvent)event
                   userList:(NSArray<ZIMGroupMemberInfo *> *)userList
               operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
                        groupID:(NSString *)groupID{
    switch (event) {
        case ZIMGroupMemberEventJoined:{
            for (ZIMGroupMemberInfo *memberInfo in userList) {
                [self.userList addObject:memberInfo];
            }
            break;
        }
        case ZIMGroupMemberEventLeft:{
            NSMutableArray<ZIMGroupMemberInfo *> *selecetMemberInfoList = [[NSMutableArray alloc] init];
            for(ZIMGroupMemberInfo *memberInfo in userList){

                for (ZIMGroupMemberInfo *info in self.userList) {
                    if([info.userID isEqual:memberInfo.userID]){
                        [selecetMemberInfoList addObject:info];
                    }
                }
            }
            for (ZIMGroupMemberInfo *deleteInfo in selecetMemberInfoList) {
                [self.userList removeObject:deleteInfo];
            }
            break;
        }
        case ZIMGroupMemberEventKickedout:{
            NSMutableArray<ZIMGroupMemberInfo *> *selecetMemberInfoList = [[NSMutableArray alloc] init];
            for(ZIMGroupMemberInfo *memberInfo in userList){

                for (ZIMGroupMemberInfo *info in self.userList) {
                    if([info.userID isEqual:memberInfo.userID]){
                        [selecetMemberInfoList addObject:info];
                    }
                }
            }
            for (ZIMGroupMemberInfo *deleteInfo in selecetMemberInfoList) {
                [self.userList removeObject:deleteInfo];
            }
            break;
        }
        case ZIMGroupMemberEventInvited:{
            for (ZIMGroupMemberInfo *memberInfo in userList) {
                [self.userList addObject:memberInfo];
            }
            break;
        }
            
        default:
            break;
    }
    [self.groupMemberListTableView reloadData];
}

- (void)groupMemberInfoUpdated:(NSArray<ZIMGroupMemberInfo *> *)userInfo
              operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
                       groupID:(NSString *)groupID{
    if(groupID != self.myGroupID){
        return;
    }
    for (ZIMGroupMemberInfo *info in userInfo) {
        if([info.userID isEqual:[ZGZIMManager shared].myUserID]){
            if(info.memberRole == 1){
                self.isGroupOwner = true;
            }else{
                self.isGroupOwner = false;
            }
        }
    }
    [self.groupMemberListTableView reloadData];
}

- (void)receiveGroupMessage:(NSArray<ZIMMessage *> *)messageList
                fromGroupID:(NSString *)fromGroupID{
    if(![fromGroupID isEqual:self.myGroupInfo.groupID]) return;
    self.unreadMessageCount = self.unreadMessageCount + 1;
    self.groupNameLabel.text= [NSString stringWithFormat:@"%@(%u)",NSLocalizedString(@"Unread Message",nil),self.unreadMessageCount];
}


    

@end
