//
//  transferGroupOwnerViewController.m
//  ZIMExampleDemo
//
//  Created by 武耀琳 on 2022/2/20.
//

#import "TransferGroupOwnerViewController.h"
#import "TransferGroupOwnerGroupMumberListTableViewCell.h"
#import "GroupSetViewController.h"
#import "ZGZIMManager.h"
#import "ZGHUDHelper.h"
#import "SelectUserIDDataSource.h"
#import "MJRefresh.h"
@interface TransferGroupOwnerViewController ()<UITableViewDelegate,UITableViewDataSource,userIDSelectedNotification,ZIMEventDelegate>
@property (weak, nonatomic) IBOutlet UILabel *unreadMessageLabel;

@property unsigned int nextFlag;

@property NSMutableArray<ZIMGroupMemberInfo *> * _Nonnull userList;

@property (weak, nonatomic) IBOutlet UITableView *GroupMemberListTableView;

@property (nonatomic) NSString *selectedUserID;
@property (weak, nonatomic) IBOutlet UIButton *okButton;

@end

@implementation TransferGroupOwnerViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [[SelectUserIDDataSource shared] addDelegate:self];
    [self queryGroupMemberList];
    [[ZGZIMManager shared] addZIMEventDelegate:self];
    self.unreadMessageLabel.text = [NSString stringWithFormat:@"%@(%u)",NSLocalizedString(@"Unread Message",nil),self.masterVC.unreadMessageCount];
    [self.okButton setTitle:NSLocalizedString(@"OK", nil) forState:UIControlStateNormal];
}
-(void)queryGroupMemberList{
    ZIMGroupMemberQueryConfig *config = [[ZIMGroupMemberQueryConfig alloc] init];
    config.count = 20;
    config.nextFlag = self.nextFlag;
    [[ZGZIMManager shared] queryGroupMemberListByGroupID:self.myGroupID config:config callback:^(NSString * _Nonnull groupID, NSArray<ZIMGroupMemberInfo *> * _Nonnull userList, unsigned int nextFlag, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code != 0){
            return;
        }
        if(self.userList == nil){
            self.userList = (NSMutableArray *)userList;
        }
        else{
            for (ZIMGroupMemberInfo *info in userList) {
                [self.userList addObject:info];
            }
            self.nextFlag = nextFlag;
        }
        [self.GroupMemberListTableView reloadData];
    }];
}

- (IBAction)backButtonClicked:(UIButton *)sender {

    [self.view.window setRootViewController:self.masterVC];
    [SelectUserIDDataSource releaseMyself];
}

- (IBAction)confirmChangeButtonClicked:(UIButton *)sender {
    [[ZGZIMManager shared] transferGroupOwner:self.selectedUserID groupID:self.myGroupID callback:^(NSString * _Nonnull groupID, NSString * _Nonnull toUserID, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code == 0){
            [ZGHUDHelper showMessage:@"success"];
            self.masterVC.isGroupOwner = false;
            [self.masterVC reloadMyTableView];
            [self.masterVC hiddenGroupOwnerItem];
        }
        else{
            [ZGHUDHelper showMessage:[NSString stringWithFormat:@"failed!error code:%d,error message:%@",(int)errorInfo.code,errorInfo.message]];
        }
    }];
}
//MARK: - TableViewDelegate
- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {

    return self.userList.count;
}
- (nonnull UITableViewCell *)tableView:(nonnull UITableView *)tableView cellForRowAtIndexPath:(nonnull NSIndexPath *)indexPath {
    TransferGroupOwnerGroupMumberListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"TransferGroupOwnerGroupMumberListTableViewCell"];
    if(self.userList[indexPath.row].memberNickname != nil &&![self.userList[indexPath.row].memberNickname isEqual:@""]){
        cell.GroupMemberName.text = self.userList[indexPath.row].memberNickname;
    }else if (self.userList[indexPath.row].userName != nil &&![self.userList[indexPath.row].userName isEqual:@""]){
        cell.GroupMemberName.text = self.userList[indexPath.row].userName;
    }else{
        cell.GroupMemberName.text = self.userList[indexPath.row].userID;
    }
    cell.userID = self.userList[indexPath.row].userID;
    //cell.masterVc = self;
    return cell;
}
//MARK: -
-(void)useridSelected:(NSString *)userID{
    self.selectedUserID = userID;
}


- (void)conversationTotalUnreadMessageCountUpdated:(unsigned int)totalUnreadMessageCount{
    self.unreadMessageLabel.text = [NSString stringWithFormat:@"%@(%u)",NSLocalizedString(@"Unread Message",nil),totalUnreadMessageCount];
}
@end
