//
//  GroupListViewController.m
//  ZIMExampleDemo
//
//  Created by 武耀琳 on 2022/1/19.
//

#import "GroupListViewController.h"
#import "GroupListTableViewCell.h"
#import "ZGZIMManager.h"
#import "ZGPopMenuView.h"
#import "CreatePeerViewController.h"
#import "CreateGroupViewController.h"
#import "JoinGroupViewController.h"
#import "JoinRoomViewController.h"
#import "LoginViewController.h"
#import "ConversationListDataSource.h"
#import "ZGHUDHelper.h"
#import "MJRefresh.h"
@interface GroupListViewController ()<UITableViewDelegate,UITableViewDataSource>

@property (weak, nonatomic) IBOutlet UITableView *groupListTableView;
@property NSArray<ZIMGroup *> *groupInfoList;

@end

@implementation GroupListViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.groupListTableView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
    [self queryGroupList];
    self.groupListTableView.mj_header = [MJRefreshNormalHeader headerWithRefreshingBlock:^{
        [self queryGroupList];
        
    }];
}

-(void)queryGroupList{
    if(!self.groupInfoList){
        self.groupInfoList = [[NSMutableArray alloc] init];
    }
    [[ZGZIMManager shared] queryGroupList:^(NSArray<ZIMGroup *> * _Nonnull groupList, ZIMError * _Nonnull errorInfo) {
        if(errorInfo.code != 0){
            [ZGHUDHelper showMessage:[NSString stringWithFormat:@"failed，error code：%d,error message:%@",(int)errorInfo.code,errorInfo.message]];
            return;
        }
        self.groupInfoList = groupList;
        [self.groupListTableView reloadData];
        [self.groupListTableView.mj_header endRefreshing];
    }];
}

//MARK: - TableViewDelegate
- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    return _groupInfoList.count;
}

- (nonnull UITableViewCell *)tableView:(nonnull UITableView *)tableView cellForRowAtIndexPath:(nonnull NSIndexPath *)indexPath {
    GroupListTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"GroupListTableViewCell"];
    [cell.groupIDLabel setText:[NSString stringWithFormat:@"%@",_groupInfoList[indexPath.row].baseInfo.groupID]];
    [cell.groupNameLabel setText:_groupInfoList[indexPath.row].baseInfo.groupName];
    cell.masterVc = (id)self;
    if(![_groupInfoList[indexPath.row].baseInfo.groupAvatarUrl isEqual:@""]){
        [cell.groupAvatarImageView setImage:[UIImage imageWithData:[NSData dataWithContentsOfURL:[NSURL URLWithString:_groupInfoList[indexPath.row].baseInfo.groupAvatarUrl]]]];

    }else{
        [cell.groupAvatarImageView setImage:[UIImage imageNamed:@"people"]];
    }
    
    return cell;
}


@end
