//
//  LoginViewController.m
//  ZIMExampleDemo
//
//  Created by Zego on 2022/1/19.
//

#import "LoginViewController.h"
#import "KeyboardToolBar.h"
#import "ZGZIMManager.h"
#import "KeyCenter.h"
#import "ZGHUDHelper.h"
#import "ConversationListDataSource.h"
#import "LoginSettingViewController.h"
#include <memory>
#include<string>
#include <vector>
@interface LoginViewController ()
@property (weak, nonatomic) IBOutlet UITextField *userIDTextField;
@property (weak, nonatomic) IBOutlet UITextField *userNameTextField;
@property NSString *userID;
@property NSString *userName;
@property NSArray<ZIMConversation *> *myConversationList;
@property (weak, nonatomic) IBOutlet UIButton *loginButton;

@property UITextField *tokenTextField;

@end

@implementation LoginViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"set-2"] style:UIBarButtonItemStyleDone target:self action:@selector(setButtonClicked)];
    [self.navigationController setNavigationBarHidden:NO];
    
    [KeyboardToolBar registerKeyboardToolBarWithTextField:_userIDTextField];
    self.userIDTextField.text = NSLocalizedString(@"Required", nil);
    UIColor *color = [UIColor grayColor];
    [self.userIDTextField setTextColor:color];
    self.userNameTextField.text = NSLocalizedString(@"Optional", nil);
    [self.userNameTextField setTextColor:color];
    [self.loginButton setTitle:NSLocalizedString(@"login", nil) forState:UIControlStateNormal];
}

-(void)setButtonClicked{
    LoginSettingViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"LoginSettingViewController"];
    [self.navigationController pushViewController:vc animated:YES];
}
//MARK: - UserIDTextField
- (IBAction)userIDTextFieldDidBegin:(UITextField *)sender {
    NSLocalizedString(@"Required", nil);
    if([sender.text isEqual:NSLocalizedString(@"Required", nil)]){
        sender.text = @"";
        UIColor *color = [UIColor blackColor];
        [sender setTextColor:color];
    }
}

- (IBAction)userIDTextFieldDidChanged:(UITextField *)sender {
    
    if(![sender.text isEqual:NSLocalizedString(@"Required", nil)]){
        self.userID = sender.text;
    }
    
}

- (IBAction)userIDTextFieldDidEnd:(UITextField *)sender {
    if([sender.text isEqual:@""]){
        sender.text = NSLocalizedString(@"Required", nil);
        UIColor *color = [UIColor grayColor];
        [sender setTextColor:color];
    }
}


- (IBAction)userNameTextFieldDidBegin:(UITextField *)sender {
    if([sender.text isEqual:NSLocalizedString(@"Optional", nil)]){
        sender.text = @"";
        UIColor *color = [UIColor blackColor];
        [sender setTextColor:color];
    }
}
- (IBAction)userNameTextFieldDidChanged:(UITextField *)sender {
    if(![sender.text isEqual:NSLocalizedString(@"Optional", nil)]){
        self.userName = sender.text;
    }
}


- (IBAction)userNameTextFieldDidEnd:(UITextField *)sender {
    if([sender.text isEqual:@""]){
        sender.text = NSLocalizedString(@"Optional", nil);
        UIColor *color = [UIColor grayColor];
        [sender setTextColor:color];
    }
}
//MARK: - SettingButton
- (IBAction)settingButtonClicked:(UIButton *)sender {
    LoginSettingViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"LoginSettingViewController"];
    [self.navigationController pushViewController:vc animated:YES];
    
}
//MARK: - LoginButton
- (IBAction)loginButtonClicked:(UIButton *)sender {
    if([self.userID isEqual: @""]||self.userID ==nil){
        [ZGHUDHelper showMessage:NSLocalizedString(@"It can only contain letters and numbers.", nil)];
        return;
    }
    
    if(KeyCenter.isUseToken){
        UIAlertController *alertView = [UIAlertController alertControllerWithTitle:@"tips" message:@"please input token" preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            if(self.tokenTextField == nil||[self.tokenTextField.text isEqual:@""] ||self.tokenTextField.text == nil){
                [ZGHUDHelper showMessage:@"The token cannot be null"];
            }else{
                [self beginLogin:self.tokenTextField.text];
            }
            
        }];
            [alertView addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
            self.tokenTextField = textField;
        }];
        [alertView addAction:action];
        [self presentViewController:alertView animated:YES completion:nil];
    }else{
        [self beginLogin:@""];
    }
}

-(void)beginLogin:(NSString *)token{
    [ZGHUDHelper showNetworkLoading];
    ZIMAppConfig *appConfig = [[ZIMAppConfig alloc] init];
    appConfig.appID = KeyCenter.appID;
    appConfig.appSign = KeyCenter.appSign;
    [[ZGZIMManager shared] createZIM:appConfig];
   
    ZIMUserInfo *userInfo = [[ZIMUserInfo alloc] init];
    userInfo.userID = self.userID;
    userInfo.userName = self.userName;
    [[ZGZIMManager shared] login:userInfo token:token callback:^(ZIMError * _Nonnull errorInfo) {
          [ZGHUDHelper hideNetworkLoading];
          if (errorInfo.code == 0) {
              [self SaveLoggedinCount:self.userID userName:self.userName];
              
              [self loginSuccess];
          }else{
              [ZGHUDHelper showMessage:[NSString stringWithFormat:@"Login failed! ErrorCode: %d,%@", (int)errorInfo.code,errorInfo.message]];
              NSLog(@"[510]Login failed! ErrorCode: %d,%@", (int)errorInfo.code,errorInfo.message);
              [[ZGZIMManager shared] destroyZIM];
          }
      }
    ];
}

-(void)loginSuccess{
    UITabBarController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"TabBarViewController"];
    [ZGZIMManager shared].myTabbar = (id)vc;
    NSMutableArray *controllers = [NSMutableArray arrayWithArray:self.navigationController.viewControllers];
    controllers[0] = vc;
    [self.navigationController setViewControllers:controllers];
    [[ConversationListDataSource shared] queryMoreConversationList];
}


//MARK: - Common
-(void)SaveLoggedinCount:(NSString *)userID
                userName:(NSString *)userName{
    NSUserDefaults *userDefault = [NSUserDefaults standardUserDefaults];
    [userDefault setObject:userID forKey:@"alreadyUserID"];
    [userDefault setObject:userName forKey:@"alreadyUserName"];
  //  NSDate
}



@end
