//
//  ZGRoomZIMManager.m
//  ZIMExampleLegacy
//
//  Created by Patrick Fu on 2021/7/5.
//  Copyright © 2021 Zego. All rights reserved.
//

#import "ZGZIMManager.h"
#import "GHConsole.h"
#import "NSArray+Log.h"
#import "NSDictionary+MyLog.h"
#import <ZIM/ZIMEventHandler.h>
#import "AppDelegate.h"
#import "ZGHUDHelper.h"
#import "LoginViewController.h"
#import "ConversationListDataSource.h"
#import "NSDictionary+safeInvoke.h"
#import "NSMutableArray+safeInvoke.h"
#import "NSMutableDictionary+safeInvoke.h"
#import "NSObject+safeInvoke.h"
#import "CallInviteView.h"
#import "KeyCenter.h"
#import "AppDelegate.h"
#import "ZIMPluginConverter.h"
static ZGZIMManager *_sharedManager = nil;

@interface ZGZIMManager () <ZIMEventHandler>


@property UITextField *tokenTextField;

@property (nonatomic, strong) NSLock *lock;

@property (nonatomic, strong) NSHashTable<id<ZIMEventDelegate>> *zimEventDelegates;

@end

@implementation ZGZIMManager

+ (ZGZIMManager *)shared {
    if (!_sharedManager) {
        @synchronized (self) {
            if (!_sharedManager) {
                _sharedManager = [[self alloc] init];
                _sharedManager.lock = [[NSLock alloc] init];
                _sharedManager.zimEventDelegates = [NSHashTable weakObjectsHashTable];
            }
        }
    }
    return _sharedManager;
}

- (instancetype)init
{
    self = [super init];
    if (self) {
    }
    return self;
}


-(void)addZIMEventDelegate:(id<ZIMEventDelegate>)delegate{
    [self.zimEventDelegates addObject:delegate];
}

-(void)removeZIMEventDelegate:(id<ZIMEventDelegate>)delegate{
    [self.zimEventDelegates removeObject:delegate];
}


//MARK: - MAIN
- (void)setLogConfig:(NSString*)logPath
             logSize:(int)logSize{
    ZIMLogConfig *zimLogConfig = [[ZIMLogConfig alloc]init];
    zimLogConfig.logPath = logPath;
    zimLogConfig.logSize = logSize;
    [ZIM setLogConfig:zimLogConfig];
}

- (void)createZIM:(ZIMAppConfig *)config{
    if([ZIM getInstance] != nil){
        [[ZIM getInstance] destroy];
    }
    [ZIM createWithAppConfig:config];
    GGLog(@"[GGLog][operation][createWithAppConfig] appID:%u",config.appID);
    [self setEventHandler];
}

- (void)setEventHandler{
    [[ZIM getInstance] setEventHandler:self];
    GGLog(@"[GGLog][operation][setEventHandler]");
}

- (void)destroyZIM {
    
    [[ZIM getInstance] destroy];
    GGLog(@"[GGLog][operation][destroy]");
}


- (void)login:(ZIMUserInfo *)userInfo
        token:(NSString *)token
     callback:(ZIMLoggedInCallback)callback{
    [[ZIM getInstance] loginWithUserInfo:userInfo token:token callback:^(ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMLoggedInCallback] error code:%lu,error message:%@",errorInfo.code,errorInfo.message);
       // [AppDelegate voipRegistration];
        callback(errorInfo);
    }];
    GGLog(@"[GGLog][operation][loginWithUserInfo] userID:%@,userName:%@,token:%@",userInfo.userID,userInfo.userName,token);
    self.myUserID = userInfo.userID;
}


- (void)logout {
    [[ZIM getInstance] logout];
    GGLog(@"[GGLog][operation][logout]");
}

- (void)renewToken:(NSString *)token
          callback:(ZIMTokenRenewedCallback)callback{
    [[ZIM getInstance] renewToken:token callback:^(NSString * _Nonnull token, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMTokenRenewedCallback] token:%@, error code:%lu,error message:%@",token,errorInfo.code,errorInfo.message);
        callback(token,errorInfo);
    }];
    GGLog(@"[GGLog][operation][renewToken] token:%@",token);
}

- (void)uploadLog:(ZIMLogUploadedCallback)callback{
    [[ZIM getInstance] uploadLog:^(ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMLogUploadedCallback] error code:%lu,error message:%@",errorInfo.code,errorInfo.message);
        callback(errorInfo);
    }];
    GGLog(@"[GGLog][operation][uploadLog]");
}

- (void)queryUsersInfo:(NSArray<NSString *> *)userIDs
                config:(ZIMUsersInfoQueryConfig *)config
              callback:(ZIMUsersInfoQueriedCallback)callback{
    [[ZIM getInstance] queryUsersInfo:userIDs config:config callback:^(NSArray<ZIMUserFullInfo *> * _Nonnull userList, NSArray<ZIMErrorUserInfo *> * _Nonnull errorUserList, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMUsersInfoQueriedCallback] error code:%lu,error message:%@,userList count:%lu,error userList count :%lu",errorInfo.code,errorInfo.message,userList.count,errorUserList.count);
        callback(userList,errorUserList,errorInfo);
    }];
    GGLog(@"[GGLog][operation][queryUsersInfo], userID count:%lu,isQueryFromServer:%@",userIDs.count,config.isQueryFromServer?@"true":@"false");
}

- (void)updateUserName:(NSString *)userName callback:(ZIMUserNameUpdatedCallback)callback{
    [[ZIM getInstance] updateUserName:userName callback:^(NSString * _Nonnull userName, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMUserNameUpdatedCallback] error code:%lu,error message:%@,user name:%@",errorInfo.code,errorInfo.message,userName);
        callback(userName,errorInfo);
    }];
    GGLog(@"[GGLog][operation][updateUserName], userName:%@",userName);
}

- (void)updateUserExtendedData:(NSString *)userExtendedData
                      callback:(ZIMUserExtendedDataUpdatedCallback)callback{
    [[ZIM getInstance] updateUserExtendedData:userExtendedData callback:^(NSString * _Nonnull extendedData, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMUserExtendedDataUpdatedCallback] error code:%lu,error message:%@, extendedData:%@",errorInfo.code,errorInfo.message,userExtendedData);
        callback(extendedData,errorInfo);
    }];
    GGLog(@"[GGLog][operation][updateUserExtendedData], userExtendedData:%@",userExtendedData);
}

- (void)updateUserAvatarUrl:(NSString *)userAvatarUrl
                   callback:(ZIMUserAvatarUrlUpdatedCallback)callback{
    [[ZIM getInstance] updateUserAvatarUrl:userAvatarUrl callback:^(NSString * _Nonnull userAvatarUrl, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMUserAvatarUrlUpdatedCallback] error code:%lu,error message:%@, userAvatarUrl:%@",errorInfo.code,errorInfo.message,userAvatarUrl);
        callback(userAvatarUrl,errorInfo);
    }];
    GGLog(@"[GGLog][operation][updateUserAvatarUrl], userAvatarUrl:%@",userAvatarUrl);
}
//MARK: - MESSAGE
- (void)sendPeerMessage:(ZIMMessage *)message
               toUserID:(NSString *)toUserID
                 config:(ZIMMessageSendConfig *)config
               callback:(ZIMMessageSentCallback)callback{
    
    [self sendMessage:message toConversationID:toUserID conversationType:ZIMConversationTypePeer config:config notification:nil callback:callback];
}

- (void)sendMessage:(ZIMMessage *)message
    toConversationID:(NSString *)toConversationID
    conversationType:(ZIMConversationType)conversationType
              config:(ZIMMessageSendConfig *)config
        notification:(nullable ZIMMessageSendNotification *)notification
           callback:(ZIMMessageSentCallback)callback{
    message.extendedData = @"510-extenedData";
    [[ZIM getInstance] sendMessage:message toConversationID:toConversationID conversationType:conversationType config:config notification:nil callback:^(ZIMMessage * _Nonnull message, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMMessageSentCallback] error code:%lu,error message:%@,sent status:%lu, message extendedData:%@",errorInfo.code,errorInfo.message,message.sentStatus,message.extendedData);
        callback(message,errorInfo);
    }];
    GGLog(@"[GGLog][operation][sendMessage], toConversationID:%@,conversationType:%lu,hasReceipt:%@,message extendedData:%@,resource id:%@",toConversationID,conversationType,config.hasReceipt?@"true":@"false",message.extendedData,config.pushConfig.resourcesID);
}


- (void)sendGroupMesage:(ZIMMessage *)message
              toGroupID:(NSString *)toGroupID
                 config:(ZIMMessageSendConfig *)config
               callback:(ZIMMessageSentCallback)callback{
    [self sendMessage:message toConversationID:toGroupID conversationType:ZIMConversationTypeGroup config:config notification:nil callback:callback];
}

- (void)sendRoomMessage:(ZIMMessage *)message
               toRoomID:(NSString *)toRoomID
                 config:(ZIMMessageSendConfig *)config
               callback:(ZIMMessageSentCallback)callback{
    [self sendMessage:message toConversationID:toRoomID conversationType:ZIMConversationTypeRoom config:config notification:nil callback:callback];
}

- (void)queryHistoryMessage:(NSString *)conversationID
           conversationType:(ZIMConversationType)conversationType
                     config:(ZIMMessageQueryConfig *)config
                   callback:(ZIMMessageQueriedCallback)callback{
    
    [[ZIM getInstance] queryHistoryMessageByConversationID:conversationID conversationType:conversationType config:config callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, NSArray<ZIMMessage *> * _Nonnull messageList, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMMessageQueriedCallback] error code:%lu,error message:%@,conversationID:%@,conversationType:%lu,messagelist count:%lu",errorInfo.code,errorInfo.message,conversationID,conversationType,messageList.count);
        for(ZIMMessage *message in messageList){
            GGLog(@"[GGLog][callback][ZIMMessageQueriedCallback]loop:message id:%ld,message extended data:%@",message.messageID,message.extendedData);
        }
        callback(conversationID,conversationType,messageList,errorInfo);
    }];
    GGLog(@"[GGLog][operation][queryHistoryMessage], conversationID:%@,conversationType:%lu,count:%d",conversationID,(unsigned long)conversationType,config.count);
}

- (void)deleteMessageByConversationID:(NSString *)conversationID
                     conversationType:(ZIMConversationType)conversationType
                               config:(ZIMMessageDeleteConfig *)config
                             callback:(ZIMMessageDeletedCallback)callback{
    [[ZIM getInstance] deleteAllMessageByConversationID:conversationID conversationType:conversationType config:config callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMMessageDeletedCallback] error code:%lu,error message:%@,conversationID:%@,conversationType:%lu",errorInfo.code,errorInfo.message,conversationID,conversationType);
        callback(conversationID,conversationType,errorInfo);
    }];
    GGLog(@"[GGLog][operation][deleteMessageByConversationID], conversationID:%@,conversationType:%lu",conversationID,(unsigned long)conversationType);
}

- (void)deleteMessages:(NSArray<ZIMMessage *> *)messageList
        conversationID:(NSString *)conversationID
      conversationType:(ZIMConversationType)conversationType
                config:(ZIMMessageDeleteConfig *)config
              callback:(ZIMMessageDeletedCallback)callback{
    [[ZIM getInstance] deleteMessages:messageList conversationID:conversationID conversationType:conversationType config:config callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMMessageDeletedCallback] error code:%lu,error message:%@,conversationID:%@,conversationType:%lu",errorInfo.code,errorInfo.message,conversationID,conversationType);
        callback(conversationID,conversationType,errorInfo);
    }];
    GGLog(@"[GGLog][operation][deleteMessages],message list count:%lu,conversationID:%@,conversationType:%lu",messageList.count,conversationID,(unsigned long)conversationType);
}


- (void)sendMediaMessage:(ZIMMediaMessage *)message
        toConversationID:(NSString *)toConversationID
        conversationType:(ZIMConversationType)conversationType
                  config:(ZIMMessageSendConfig *)config
                progress:(ZIMMediaUploadingProgress)progress
                callback:(ZIMMessageSentCallback)callback{
    message.extendedData = @"510-extenedData";
    ZIMMediaMessageSendNotification *notification = [[ZIMMediaMessageSendNotification alloc] init];
    notification.onMediaUploadingProgress = ^(ZIMMediaMessage * _Nonnull message, unsigned long long currentFileSize, unsigned long long totalFileSize) {
        GGLog(@"[GGLog][callback][ZIMMediaUploadingProgress] current file size:llu:%llu, total file size:%llu ",currentFileSize,totalFileSize);
        progress(message,currentFileSize,totalFileSize);
    };
    [[ZIM getInstance] sendMediaMessage:message toConversationID:toConversationID conversationType:conversationType config:config notification:notification callback:^(ZIMMessage * _Nonnull message, ZIMError * _Nonnull errorInfo) {
        callback(message,errorInfo);
        if(message.type == ZIMMessageTypeImage){
            ZIMImageMessage *imageMsg = (ZIMImageMessage *)message;
            GGLog(@"[GGLog][callback][ZIMMessageSentCallback] error code:%lu,error message:%@,orignal height:%f,orignal width:%f ,large height:%f,large,width:%f,thumbnail height:%f,thumbnail width:%f,message extended data:%@",errorInfo.code,errorInfo.message,imageMsg.originalImageSize.height,imageMsg.originalImageSize.width,imageMsg.largeImageSize.height,imageMsg.largeImageSize.width,imageMsg.thumbnailSize.height,imageMsg.thumbnailSize.width,message.extendedData);
        }else if (message.type == ZIMMessageTypeVideo){
            ZIMVideoMessage *videoMsg = (ZIMVideoMessage *)message;
            GGLog(@"[GGLog][callback][ZIMMessageSentCallback] error code:%lu,error message:%@,first frame height:%f,first frame width:%f ,message extended data:%@",errorInfo.code,errorInfo.message,videoMsg.videoFirstFrameSize.height,videoMsg.videoFirstFrameSize.width,message.extendedData);
        }
        else{
            GGLog(@"[GGLog][callback][ZIMMessageSentCallback] error code:%lu,error message:%@,message extended data:%@",errorInfo.code,errorInfo.message,message.extendedData);
        }
    }];
    GGLog(@"[GGLog][operation][sendMediaMessage],toConversationID:%@,conversationType:%lu,message extended data:%@",toConversationID,conversationType,message.extendedData);
}

- (void)downloadMediaFileWithMessage:(ZIMMediaMessage *)message
                            fileType:(ZIMMediaFileType)fileType
                            progress:(ZIMMediaDownloadingProgress)progress
                            callback:(ZIMMediaDownloadedCallback)callback{
    [[ZIM getInstance] downloadMediaFileWithMessage:message fileType:fileType progress:^(ZIMMediaMessage * _Nonnull message, unsigned long long currentFileSize, unsigned long long totalFileSize) {
        progress(message,currentFileSize,totalFileSize);
        GGLog(@"[GGLog][ZIMMediaDownloadingProgress] current file size:llu:%llu, total file size:%llu ",currentFileSize,totalFileSize);
    } callback:^(ZIMMediaMessage * _Nonnull message, ZIMError * _Nonnull errorInfo) {
        callback(message,errorInfo);
        if(message.type == ZIMMessageTypeImage){
            ZIMImageMessage *imageMsg = (ZIMImageMessage *)message;
            GGLog(@"[GGLog][callback][ZIMMediaDownloadedCallback] error code:%lu,error message:%@,orignal height:%f,orignal width:%f ,large height:%f,large,width:%f,thumbnail height:%f,thumbnail width:%f",errorInfo.code,errorInfo.message,imageMsg.originalImageSize.height,imageMsg.originalImageSize.width,imageMsg.largeImageSize.height,imageMsg.largeImageSize.width,imageMsg.thumbnailSize.height,imageMsg.thumbnailSize.width);
        }else if (message.type == ZIMMessageTypeVideo){
            ZIMVideoMessage *videoMsg = (ZIMVideoMessage *)message;
            GGLog(@"[GGLog][callback][ZIMMediaDownloadedCallback] error code:%lu,error message:%@,first frame height:%f,first frame width:%f ",errorInfo.code,errorInfo.message,videoMsg.videoFirstFrameSize.height,videoMsg.videoFirstFrameSize.width);
        }
        else{
            GGLog(@"[GGLog][callback][ZIMMediaDownloadedCallback] error code:%lu,error message:%@",errorInfo.code,errorInfo.message);
        }
    }];
    GGLog(@"[GGLog][operation][downloadMediaFileWithMessage],file type:%lu",fileType);
}

- (void)revokeMessage:(ZIMMessage *)message
               config:(ZIMMessageRevokeConfig *)config
             callback:(ZIMMessageRevokedCallback)callback{
    [[ZIM getInstance] revokeMessage:message config:config callback:^(ZIMMessage * _Nonnull message, ZIMError * _Nonnull errorInfo) {
        callback(message,errorInfo);
        GGLog(@"[GGLog][callback][ZIMMessageRevokedCallback],code:%ld,message:%@",errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][revokeMessage]");
}

- (void)sendMessageReceiptsRead:(NSArray<ZIMMessage *> *)messageList
                 conversationID:(NSString *)conversationID
               conversationType:(ZIMConversationType)conversationType
                       callback:(ZIMMessageReceiptsReadSentCallback)callback{
    [[ZIM getInstance] sendMessageReceiptsRead:messageList conversationID:conversationID conversationType:conversationType callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, NSArray<NSNumber *> * _Nonnull errorMessageIDs, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMMessageReceiptsReadSentCallback],code:%ld,message:%@",errorInfo.code,errorInfo.message);
        callback(conversationID,conversationType,errorMessageIDs,errorInfo);
    }];
    GGLog(@"[GGLog][operation][sendMessageReceiptsRead]");
}

- (void)queryMessageReceiptsInfoByMessageList:(NSArray<ZIMMessage *> *)messageList
                               conversationID:(NSString *)conversationID
                             conversationType:(ZIMConversationType)conversationType
                                     callback:(ZIMMessageReceiptsInfoQueriedCallback)callback{
    [[ZIM getInstance] queryMessageReceiptsInfoByMessageList:messageList conversationID:conversationID conversationType:conversationType callback:^(NSArray<ZIMMessageReceiptInfo *> * _Nonnull infos, NSArray<NSString *> * _Nonnull errorMessageIDs, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMMessageReceiptsInfoQueriedCallback],code:%ld,message:%@",errorInfo.code,errorInfo.message);
        for (ZIMMessageReceiptInfo *info in infos) {
            GGLog(@"messageID:%lld,isSelfOperated:%@",info.messageID,info.isSelfOperated?@"true":@"false");
        }
        callback(infos,errorMessageIDs,errorInfo);
    }];
    GGLog(@"[GGLog][operation][queryMessageReceiptsInfoByMessageList]");
}
//MARK: - Conversation
- (void)queryConversationListWithConfig:(ZIMConversationQueryConfig *)config
                               callback:(ZIMConversationListQueriedCallback)callback{
    [[ZIM getInstance] queryConversationListWithConfig:config callback:^(NSArray<ZIMConversation *> * _Nonnull conversationList, ZIMError * _Nonnull errorInfo) {
        callback(conversationList,errorInfo);
        GGLog(@"[GGLog][callback][ZIMConversationListQueriedCallback] error code:%lu,error message:%@,conversationList count:%lu",errorInfo.code,errorInfo.message,(unsigned long)conversationList.count);
    }];
    GGLog(@"[GGLog][operation][queryConversationListWithConfig],count:%u",config.count);
}

- (void)deleteConversation:(NSString *)conversationID
                      type:(ZIMConversationType)type
                    config:(ZIMConversationDeleteConfig *)config
                  callback:(ZIMConversationDeletedCallback)callback{
    [[ZIM getInstance] deleteConversation:conversationID conversationType:type config:config callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {
        callback(conversationID,conversationType,errorInfo);
        GGLog(@"[GGLog][callback][ZIMConversationDeletedCallback] error code:%lu,error message:%@,conversationID:%@,conversationType:%lu",errorInfo.code,errorInfo.message,conversationID,(unsigned long)conversationType);
    }];
    GGLog(@"[GGLog][operation][deleteConversation],conversationID:%@,type:%lu",conversationID,type);
}

- (void)clearConversationUnreadMessageCount:(NSString *)conversationID
                                       type:(ZIMConversationType)type
                                   callback:
(ZIMConversationUnreadMessageCountClearedCallback)callback{
    [[ZIM getInstance] clearConversationUnreadMessageCount:conversationID conversationType:type callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMConversationUnreadMessageCountClearedCallback] error code:%lu,error message:%@,conversationID:%@,conversationType:%lu",errorInfo.code,errorInfo.message,conversationID,(unsigned long)conversationType);
        callback(conversationID,conversationType,errorInfo);
    }];
    GGLog(@"[GGLog][operation][clearConversationUnreadMessageCount],conversationID:%@,type:%lu",conversationID,type);
}

- (void)sendConversationMessageReceiptRead:(NSString *)conversationID
                          conversationType:(ZIMConversationType)conversationType
                                  callback:(ZIMConversationMessageReceiptReadSentCallback)callback{
    [[ZIM getInstance] sendConversationMessageReceiptRead:conversationID conversationType:conversationType callback:^(NSString * _Nonnull conversationID, ZIMConversationType conversationType, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMConversationMessageReceiptReadSentCallback] error code:%lu,error message:%@,conversationID:%@,conversationType:%lu",errorInfo.code,errorInfo.message,conversationID,conversationType);
        callback(conversationID,conversationType,errorInfo);
    }];
    GGLog(@"[GGLog][operation][sendConversationMessageReceiptRead] conversationID:%@,conversationType:%lu",conversationID,conversationType);
}
//MARK: - ROOM
- (void)createRoom:(ZIMRoomInfo *)roomInfo callback:(ZIMRoomCreatedCallback)callback {
    
    [[ZIM getInstance] createRoom:roomInfo callback:^(ZIMRoomFullInfo * _Nonnull roomInfo, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMRoomCreatedCallback] error code:%lu,error message:%@,roomID:%@,roomName:%@",errorInfo.code,errorInfo.message,roomInfo.baseInfo.roomID,roomInfo.baseInfo.roomName);
        callback(roomInfo,errorInfo);
    }];
    GGLog(@"[GGLog][operation][createRoom],roomID:%@,roomName:%@",roomInfo.roomID,roomInfo.roomName);
}

- (void)createRoom:(ZIMRoomInfo *)roomInfo config:(ZIMRoomAdvancedConfig *)config callback:(ZIMRoomCreatedCallback)callback {
    [[ZIM getInstance] createRoom:roomInfo config:config callback:^(ZIMRoomFullInfo * _Nonnull roomInfo, ZIMError * _Nonnull errorInfo) {
        callback(roomInfo,errorInfo);
        GGLog(@"[GGLog][callback][ZIMRoomCreatedCallback] error code:%lu,error message:%@,roomID:%@,roomName:%@",errorInfo.code,errorInfo.message,roomInfo.baseInfo.roomID,roomInfo.baseInfo.roomName);
    }];
    GGLog(@"[GGLog][operation][createRoom],roomID:%@,roomName:%@,roomAttributes:%@,roomDestroyDelayTime:%u",roomInfo.roomID,roomInfo.roomName,config.roomAttributes,config.roomDestroyDelayTime);
}

- (void)joinRoom:(NSString *)roomID callback:(ZIMRoomJoinedCallback)callback {
    
    [[ZIM getInstance] joinRoom:roomID callback:^(ZIMRoomFullInfo * _Nonnull roomInfo, ZIMError * _Nonnull errorInfo) {
        callback(roomInfo,errorInfo);
        GGLog(@"[GGLog][callback][ZIMRoomJoinedCallback] error code:%lu,error message:%@,roomID:%@,roomName:%@",errorInfo.code,errorInfo.message,roomInfo.baseInfo.roomID,roomInfo.baseInfo.roomName);
    }];
    GGLog(@"[GGLog][operation][joinRoom],roomID:%@",roomID);
}

- (void)leaveRoom:(NSString *)roomID callback:(ZIMRoomLeftCallback)callback {
    [[ZIM getInstance] leaveRoom:roomID callback:^(NSString * _Nonnull roomID, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMRoomLeftCallback] error code:%lu,error message:%@,roomID:%@",errorInfo.code,errorInfo.message,roomID);
        callback(roomID,errorInfo);
    }];
    GGLog(@"[GGLog][operation][leaveRoom],roomID:%@",roomID);
}

- (void)queryRoomMemberListByRoomID:(NSString *)roomID
                             config:(ZIMRoomMemberQueryConfig *)config
                           callback:(ZIMRoomMemberQueriedCallback)callback{
    [[ZIM getInstance] queryRoomMemberListByRoomID:roomID config:config callback:^(NSString * _Nonnull roomID, NSArray<ZIMUserInfo *> * _Nonnull memberList, NSString * _Nonnull nextFlag, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMRoomMemberQueriedCallback] error code:%lu,error message:%@,roomID:%@,memberlist count:%lu,nextFlag:%@",errorInfo.code,errorInfo.message,roomID,(unsigned long)memberList.count,nextFlag);
        callback(roomID,memberList,nextFlag,errorInfo);
    }];
    GGLog(@"[GGLog][operation][queryRoomMemberListByRoomID],roomID:%@,nextFlag:%@,count:%u",roomID,config.nextFlag,config.count);
}

- (void)queryRoomOnlineMemberCount:(NSString *)roomID
                          callback:(ZIMRoomOnlineMemberCountQueriedCallback)callback {
    [[ZIM getInstance] queryRoomOnlineMemberCountByRoomID:roomID callback:^(NSString * _Nonnull roomID, unsigned int count, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMRoomOnlineMemberCountQueriedCallback] error code:%lu,error message:%@,roomID:%@, count:%u,",errorInfo.code,errorInfo.message,roomID,count);
        callback(roomID,count,errorInfo);
    }];
    GGLog(@"[GGLog][operation][queryRoomOnlineMemberCount],roomID:%@",roomID);
}

- (void)setRoomAttributes:(NSDictionary<NSString *, NSString *> *)roomAttributes
                   roomID:(NSString *)roomID
                   config:(ZIMRoomAttributesSetConfig *)config
                 callback:(ZIMRoomAttributesOperatedCallback)callback {
    [[ZIM getInstance] setRoomAttributes:roomAttributes roomID:roomID config:config callback:^(NSString * _Nonnull roomID, NSArray<NSString *> * _Nonnull errorKeys, ZIMError * _Nonnull errorInfo) {
        callback(roomID,errorKeys,errorInfo);
        GGLog(@"[GGLog][callback][ZIMRoomAttributesOperatedCallback] error code:%lu,error message:%@,roomID:%@, error keys:%@,",errorInfo.code,errorInfo.message,roomID,errorKeys);
    }];
    GGLog(@"[GGLog][operation][setRoomAttributes],roomAttributes:%@,roomID:%@",roomAttributes,roomID);
}

- (void)deleteRoomAttributesByKeys:(NSArray<NSString *> *)keys
                            roomID:(NSString *)roomID
                            config:(ZIMRoomAttributesDeleteConfig *)config
                          callback:(ZIMRoomAttributesOperatedCallback)callback {
    [[ZIM getInstance] deleteRoomAttributesByKeys:keys roomID:roomID config:config callback:^(NSString * _Nonnull roomID, NSArray<NSString *> * _Nonnull errorKeys, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMRoomAttributesOperatedCallback] error code:%lu,error message:%@,roomID:%@, error keys:%@,",errorInfo.code,errorInfo.message,roomID,errorKeys);
        callback(roomID,errorKeys,errorInfo);
    }];
    GGLog(@"[GGLog][operation][deleteRoomAttributesByKeys],keys:%@,roomID:%@",keys,roomID);
}

- (void)beginRoomAttributesBatchOperationWithRoomID:(NSString *)roomID
                                             config:(ZIMRoomAttributesBatchOperationConfig *)config {
    [[ZIM getInstance] beginRoomAttributesBatchOperationWithRoomID:roomID config:config];
    GGLog(@"[GGLog][operation][beginRoomAttributesBatchOperationWithRoomID],roomID:%@",roomID);
}

- (void)endRoomAttributesBatchOperationWithRoomID:(NSString *)roomID
                                         callback:(ZIMRoomAttributesBatchOperatedCallback)callback {
    [[ZIM getInstance] endRoomAttributesBatchOperationWithRoomID:roomID callback:^(NSString * _Nonnull roomID, ZIMError * _Nonnull errorInfo) {
        callback(roomID,errorInfo);
        GGLog(@"[GGLog][callback][endRoomAttributesBatchOperationWithRoomID] error code:%lu,error message:%@,roomID:%@",errorInfo.code,errorInfo.message,roomID);
    }];
    GGLog(@"[GGLog][operation][endRoomAttributesBatchOperationWithRoomID],roomID:%@",roomID);
}

- (void)queryRoomAllAttributesByRoomID:(NSString *)roomID
                              callback:(ZIMRoomAttributesQueriedCallback)callback {
    [[ZIM getInstance] queryRoomAllAttributesByRoomID:roomID callback:^(NSString * _Nonnull roomID, NSDictionary<NSString *,NSString *> * _Nonnull roomAttributes, ZIMError * _Nonnull errorInfo) {
        callback(roomID,roomAttributes,errorInfo);
        GGLog(@"[GGLog][callback][ZIMRoomAttributesQueriedCallback] error code:%lu,error message:%@,roomID:%@,roomAttributes:%@",errorInfo.code,errorInfo.message,roomID,roomAttributes);
    }];
    GGLog(@"[GGLog][operation][queryRoomAllAttributesByRoomID],roomID:%@",roomID);
}
//MARK: - Group
- (void)createGroup:(ZIMGroupInfo *)groupInfo
           userIDs:(NSArray<NSString *> *)userIDs
           callback:(ZIMGroupCreatedCallback)callback{
    [[ZIM getInstance] createGroup:groupInfo userIDs:userIDs callback:^(ZIMGroupFullInfo * _Nonnull groupInfo, NSArray<ZIMGroupMemberInfo *> * _Nonnull userList, NSArray<ZIMErrorUserInfo *> * _Nonnull errorUserList, ZIMError * _Nonnull errorInfo) {
        callback(groupInfo,userList,errorUserList,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupCreatedCallback] error code:%lu,error message:%@",errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][createGroup],groupID:%@,groupName:%@,avatar:%@,userIDs:%@",groupInfo.groupID,groupInfo.groupName,groupInfo.groupAvatarUrl,userIDs);
}

- (void)createGroup:(ZIMGroupInfo *)groupInfo
           userIDs:(NSArray<NSString *> *)userIDs
             config:(ZIMGroupAdvancedConfig *)config
           callback:(ZIMGroupCreatedCallback)callback{
    [[ZIM getInstance] createGroup:groupInfo userIDs:userIDs config:config callback:^(ZIMGroupFullInfo * _Nonnull groupInfo, NSArray<ZIMGroupMemberInfo *> * _Nonnull userList, NSArray<ZIMErrorUserInfo *> * _Nonnull errorUserList, ZIMError * _Nonnull errorInfo) {
        callback(groupInfo,userList,errorUserList,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupCreatedCallback] error code:%lu,error message:%@",errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][createGroup],groupID:%@,groupName:%@,avatar:%@,userIDs:%@",groupInfo.groupID,groupInfo.groupName,groupInfo.groupAvatarUrl,userIDs);
}

- (void)dismissGroup:(NSString *)groupID
            callback:(ZIMGroupDismissedCallback)callback{
    [[ZIM getInstance] dismissGroup:groupID callback:^(NSString * _Nonnull groupID, ZIMError * _Nonnull errorInfo) {
        callback(groupID,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupCreatedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][dismissGroup],groupID:%@",groupID);
}
- (void)joinGroup:(NSString *)groupID
         callback:(ZIMGroupJoinedCallback)callback{
    [[ZIM getInstance] joinGroup:groupID callback:^(ZIMGroupFullInfo * _Nonnull groupInfo, ZIMError * _Nonnull errorInfo) {
        callback(groupInfo,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupJoinedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupInfo.baseInfo.groupID);
    }];
    GGLog(@"[GGLog][operation][joinGroup],groupID:%@",groupID);
}

- (void)leaveGroup:(NSString *)groupID
          callback:(ZIMGroupLeftCallback)callback{
    [[ZIM getInstance] leaveGroup:groupID callback:^(NSString * _Nonnull groupID, ZIMError * _Nonnull errorInfo) {
        callback(groupID,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupJoinedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][leaveGroup],groupID:%@",groupID);
}

- (void)inviteUsersIntoGroup:(NSArray<NSString *> *)userIDList
                     groupID:(NSString *)groupID
                    callback:(ZIMGroupUsersInvitedCallback)callback{
    [[ZIM getInstance] inviteUsersIntoGroup:userIDList groupID:groupID callback:^(NSString * _Nonnull groupID, NSArray<ZIMGroupMemberInfo *> * _Nonnull userList, NSArray<ZIMErrorUserInfo *> * _Nonnull errorUserList, ZIMError * _Nonnull errorInfo) {
        callback(groupID,userList,errorUserList,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupUsersInvitedCallback] error code:%lu,error message:%@,groupID:%@,userList:%@,errorUserlist:%@",errorInfo.code,errorInfo.message,groupID,userList,errorUserList);
    }];
    GGLog(@"[GGLog][operation][inviteUsersIntoGroup],groupID:%@,userIDList:%@",groupID,userIDList);
}


- (void)kickGroupMembers:(NSArray<NSString *> *)userIDs
                 groupID:(NSString *)groupID
                callback:(ZIMGroupMemberKickedCallback)callback{
    [[ZIM getInstance] kickGroupMembers:userIDs groupID:groupID callback:^(NSString * _Nonnull groupID, NSArray<NSString *> * _Nonnull kickedUserIDList, NSArray<ZIMErrorUserInfo *> * _Nonnull errorUserList, ZIMError * _Nonnull errorInfo) {
        callback(groupID,kickedUserIDList,errorUserList,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupMemberKickedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][kickGroupMembers],groupID:%@,userIDs:%@",groupID,userIDs);
}


- (void)transferGroupOwner:(NSString *)userID
                   groupID:(NSString *)groupID
                  callback:(ZIMGroupOwnerTransferredCallback)callback{
    [[ZIM getInstance] transferGroupOwnerToUserID:userID groupID:groupID callback:^(NSString * _Nonnull groupID, NSString * _Nonnull toUserID, ZIMError * _Nonnull errorInfo) {
        callback(groupID,toUserID,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupOwnerTransferredCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][transferGroupOwner],groupID:%@,userID:%@",groupID,userID);
}

- (void)updateGroupName:(NSString *)userName
                groupID:(NSString *)groupID
               callback:(ZIMGroupNameUpdatedCallback)callback{
    
    [[ZIM getInstance] updateGroupName:userName groupID:groupID callback:^(NSString * _Nonnull groupID, NSString * _Nonnull groupName, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMGroupNameUpdatedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
        callback(groupID,groupName,errorInfo);
    }];
    GGLog(@"[GGLog][operation][updateGroupName],groupID:%@,userName:%@",groupID,userName);
}

- (void)updateGroupNotice:(NSString *)userNotice
                  groupID:(NSString *)groupID
                 callback:(ZIMGroupNoticeUpdatedCallback)callback{
    [[ZIM getInstance] updateGroupNotice:userNotice groupID:groupID callback:^(NSString * _Nonnull groupID, NSString * _Nonnull groupNotice, ZIMError * _Nonnull errorInfo) {
        callback(groupID,groupNotice,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupNoticeUpdatedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][updateGroupNotice],groupID:%@,userNotice:%@",groupID,userNotice);
}

- (void)queryGroupInfo:(NSString *)groupID
              callback:(ZIMGroupInfoQueriedCallback)callback{
    [[ZIM getInstance] queryGroupInfoByGroupID:groupID callback:^(ZIMGroupFullInfo * _Nonnull groupInfo, ZIMError * _Nonnull errorInfo) {
        callback(groupInfo,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupInfoQueriedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][queryGroupInfo],groupID:%@",groupID);
}

- (void)setGroupAttributes:(NSDictionary<NSString *, NSString *> *)groupAttributes
                   groupID:(NSString *)groupID
                  callback:(ZIMGroupAttributesOperatedCallback)callback{
    [[ZIM getInstance] setGroupAttributes:groupAttributes groupID:groupID callback:^(NSString * _Nonnull groupID, NSArray<NSString *> * _Nonnull errorKeys, ZIMError * _Nonnull errorInfo) {
        callback(groupID,errorKeys,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupAttributesOperatedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][setGroupAttributes],groupID:%@",groupID);
}

- (void)deleteGroupAttributes:(NSArray<NSString *> *)groupAttributesKeys
                      groupID:(NSString *)groupID
                     callback:(ZIMGroupAttributesOperatedCallback)callback{
    [[ZIM getInstance] deleteGroupAttributesByKeys:groupAttributesKeys groupID:groupID callback:^(NSString * _Nonnull groupID, NSArray<NSString *> * _Nonnull errorKeys, ZIMError * _Nonnull errorInfo) {
        callback(groupID,errorKeys,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupAttributesOperatedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][deleteGroupAttributes],groupID:%@",groupID);
}

- (void)queryGroupAttributes:(NSArray<NSString *> *)groupAttributesKeys
                     groupID:(NSString *)groupID
                    callback:(ZIMGroupAttributesQueriedCallback)callback{
    [[ZIM getInstance] queryGroupAttributesByKeys:groupAttributesKeys groupID:groupID callback:^(NSString * _Nonnull groupID, NSDictionary<NSString *,NSString *> * _Nonnull groupAttributes, ZIMError * _Nonnull errorInfo) {
        callback(groupID,groupAttributes,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupAttributesQueriedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][queryGroupAttributes],groupID:%@",groupID);
}

- (void)queryGroupAllAttributes:(NSString *)groupID
                       callback:(ZIMGroupAttributesQueriedCallback)callback{
    [[ZIM getInstance] queryGroupAllAttributesByGroupID:groupID callback:^(NSString * _Nonnull groupID, NSDictionary<NSString *,NSString *> * _Nonnull groupAttributes, ZIMError * _Nonnull errorInfo) {
        callback(groupID,groupAttributes,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupAttributesQueriedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][queryGroupAllAttributes],groupID:%@",groupID);
}

- (void)setGroupMemberRole:(int)role
                    userID:(NSString *)userID
                   groupID:(NSString *)groupID
                  callback:(ZIMGroupMemberRoleUpdatedCallback)callback{
    [[ZIM getInstance] setGroupMemberRole:role forUserID:userID groupID:groupID callback:^(NSString * _Nonnull groupID, NSString * _Nonnull forUserID, ZIMGroupMemberRole role, ZIMError * _Nonnull errorInfo) {
        callback(groupID,forUserID,role,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupMemberRoleUpdatedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][setGroupMemberRole],groupID:%@",groupID);
}


- (void)setGroupMemberNickname:(NSString *)nickname
                        userID:(NSString *)userID
                       groupID:(NSString *)groupID
                      callback:(ZIMGroupMemberNicknameUpdatedCallback)callback{
    [[ZIM getInstance] setGroupMemberNickname:nickname forUserID:userID groupID:groupID callback:^(NSString * _Nonnull groupID, NSString * _Nonnull forUserID, NSString * _Nonnull nickname, ZIMError * _Nonnull errorInfo) {
        callback(groupID,forUserID,nickname,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupMemberNicknameUpdatedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][setGroupMemberNickname],groupID:%@",groupID);
}

- (void)queryGroupMemberInfo:(NSString *)userID
                         groupID:(NSString *)groupID
                    callback:(ZIMGroupMemberInfoQueriedCallback)callback{
    [[ZIM getInstance] queryGroupMemberInfoByUserID:userID groupID:groupID callback:^(NSString * _Nonnull groupID, ZIMGroupMemberInfo * _Nonnull userInfo, ZIMError * _Nonnull errorInfo) {
        callback(groupID,userInfo,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupMemberInfoQueriedCallback] error code:%lu,error message:%@,groupID:%@",errorInfo.code,errorInfo.message,groupID);
    }];
    GGLog(@"[GGLog][operation][queryGroupMemberInfo],groupID:%@",groupID);
}

- (void)queryGroupList:(ZIMGroupListQueriedCallback)callback{

    [[ZIM getInstance] queryGroupList:^(NSArray<ZIMGroup *> * _Nonnull groupList, ZIMError * _Nonnull errorInfo) {
        callback(groupList,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupListQueriedCallback] error code:%lu,error message:%@",errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][queryGroupList]");
}

- (void)queryGroupMemberListByGroupID:(NSString *)groupID
                               config:(ZIMGroupMemberQueryConfig *)config
                             callback:(ZIMGroupMemberListQueriedCallback)callback{
    [[ZIM getInstance] queryGroupMemberListByGroupID:groupID config:config callback:^(NSString * _Nonnull groupID, NSArray<ZIMGroupMemberInfo *> * _Nonnull userList, unsigned int nextFlag, ZIMError * _Nonnull errorInfo) {
        callback(groupID,userList,nextFlag,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupMemberListQueriedCallback] error code:%lu,error message:%@",(unsigned long)errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][queryGroupMemberListByGroupID]");
}


- (void)queryGroupMemberCountByGroupID:(NSString *)groupID
                               callback:(ZIMGroupMemberCountQueriedCallback)callback{
    [[ZIM getInstance] queryGroupMemberCountByGroupID:groupID callback:^(NSString * _Nonnull groupID, unsigned int count, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMGroupMemberCountQueriedCallback] error code:%lu,error message:%@,count:%u",(unsigned long)errorInfo.code,errorInfo.message,count);
    }];
    GGLog(@"[GGLog][operation][queryGroupMemberCountByGroupID]");
}

- (void)queryGroupMemberInfoByUserID:(NSString *)userID
                             groupID:(NSString *)groupID
                            callback:(ZIMGroupMemberInfoQueriedCallback)callback{
    [[ZIM getInstance] queryGroupMemberInfoByUserID:userID groupID:groupID callback:^(NSString * _Nonnull groupID, ZIMGroupMemberInfo * _Nonnull userInfo, ZIMError * _Nonnull errorInfo) {
        callback(groupID,userInfo,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupMemberInfoQueriedCallback] error code:%lu,error message:%@",(unsigned long)errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][queryGroupMemberInfoByUserID]");
}

- (void)updateGroupAvatarUrl:(NSString *)groupAvatarUrl
                     groupID:(NSString *)groupID
                    callback:(ZIMGroupAvatarUrlUpdatedCallback)callback{
    [[ZIM getInstance] updateGroupAvatarUrl:groupAvatarUrl groupID:groupID callback:^(NSString * _Nonnull groupID, NSString * _Nonnull groupAvatarUrl, ZIMError * _Nonnull errorInfo) {
        callback(groupID,groupAvatarUrl,errorInfo);
        GGLog(@"[GGLog][callback][ZIMGroupAvatarUrlUpdatedCallback] error code:%lu,error message:%@",(unsigned long)errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][updateGroupAvatarUrl]");
}
// MARK: - 呼叫邀请
- (void)callInviteWithInvitees:(NSArray<NSString *> *)invitees
                        config:(ZIMCallInviteConfig *)config
                      callback:(ZIMCallInvitationSentCallback)callback{
    ZIMPushConfig *pushConfig = [[ZIMPushConfig alloc] init];
    pushConfig.title =@"title";
    pushConfig.content = @"callinvite";
    pushConfig.resourcesID = KeyCenter.resourceID;
    pushConfig.payload = @"payload";
    config.pushConfig = pushConfig;
    [[ZIM getInstance] callInviteWithInvitees:invitees config:config callback:^(NSString * _Nonnull callID, ZIMCallInvitationSentInfo * _Nonnull info, ZIMError * _Nonnull errorInfo) {
        callback(callID,info,errorInfo);
        GGLog(@"[GGLog][callback][ZIMCallInvitationSentCallback],callID:%@,timeout:%d,code:%lu,messsage:%@",callID,info.timeout,errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][callInviteWithInvitees],invittes:%@,timeout:%d,extendedData:%@,title:%@,content:%@,extendedData:%@,resourceID:%@",invitees,config.timeout,config.extendedData,config.pushConfig.title,config.pushConfig.content,config.pushConfig.payload,config.pushConfig.resourcesID);
}

-(void)callingInviteWithInvitees:(NSArray<NSString *> *)invitees
                          callID:(NSString *)callID
                          config:(ZIMCallingInviteConfig *)config
                        callback:(ZIMCallingInvitationSentCallback)callback{
    ZIMPushConfig *pushConfig = [[ZIMPushConfig alloc] init];
    pushConfig.title =@"title";
    pushConfig.content = @"calling";
    pushConfig.payload = @"payload";
    pushConfig.resourcesID = KeyCenter.resourceID;
    config.pushConfig = pushConfig;
    [[ZIM getInstance] callingInviteWithInvitees:invitees callID:callID config:config callback:^(NSString * _Nonnull callID, ZIMCallingInvitationSentInfo * _Nonnull info, ZIMError * _Nonnull errorInfo) {
        callback(callID,info,errorInfo);
        GGLog(@"[GGLog][callback][ZIMCallingInvitationSentCallback],callID:%@,code:%lu,messsage:%@",callID,errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][callingInviteWithInvitees]");
    
}

- (void)callCancelWithInvitees:(NSArray<NSString *> *)invitees
                        callID:(NSString *)callID
                        config:(ZIMCallCancelConfig *)config
                      callback:(ZIMCallCancelSentCallback)callback{
    config.extendedData = @"cancelPayload";
    ZIMPushConfig *pushConfig = [[ZIMPushConfig alloc] init];
    pushConfig.title =@"title";
    pushConfig.content = @"cancel";
    pushConfig.payload = @"payload";
    pushConfig.resourcesID = KeyCenter.resourceID;
    config.pushConfig = pushConfig;
    [[ZIM getInstance] callCancelWithInvitees:invitees callID:callID config:config callback:^(NSString * _Nonnull callID, NSArray<NSString *> * _Nonnull errorInvitees, ZIMError * _Nonnull errorInfo) {
        callback(callID,errorInvitees,errorInfo);
        GGLog(@"[GGLog][callback][ZIMCallCancelSentCallback],callID:%@,errorInvitees:%@,code:%lu,message:%@",callID,errorInvitees,errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][callCancelWithInvitees],invitees:%@,callID:%@,extendeddata:%@",invitees,callID,config.extendedData);
}


- (void)callAcceptWithCallID:(NSString *)callID
                      config:(ZIMCallAcceptConfig *)config
                    callback:(ZIMCallAcceptanceSentCallback)callback{
    config.extendedData = @"acceptPayload";
    [[ZIM getInstance] callAcceptWithCallID:callID config:config callback:^(NSString * _Nonnull callID, ZIMError * _Nonnull errorInfo) {
        callback(callID,errorInfo);
        GGLog(@"[GGLog][callback][ZIMCallAcceptanceSentCallback],callID:%@,code:%lu,message:%@",callID,errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][callAcceptWithCallID],callID:%@,extendeddata:%@",callID,config.extendedData);
}


- (void)callRejectWithCallID:(NSString *)callID
                      config:(ZIMCallRejectConfig *)config
                    callback:(ZIMCallRejectionSentCallback)callback{
    config.extendedData = @"rejectPayload";
    [[ZIM getInstance] callRejectWithCallID:callID config:config callback:^(NSString * _Nonnull callID, ZIMError * _Nonnull errorInfo) {
        callback(callID,errorInfo);
        GGLog(@"[GGLog][callback][ZIMCallRejectionSentCallback],callID:%@,code:%lu,message:%@",callID,errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][callRejectWithCallID],callID:%@,extendeddata:%@",callID,config.extendedData);
}

- (void)callQuit:(NSString *)callID
          config:(ZIMCallQuitConfig *)config
        callback:(ZIMCallQuitSentCallback)callback{
    config.extendedData = @"quitPayload";
    ZIMPushConfig *pushConfig = [[ZIMPushConfig alloc] init];
    pushConfig.title =@"title";
    pushConfig.content = @"cancel";
    pushConfig.payload = @"payload";
    pushConfig.resourcesID = KeyCenter.resourceID;
    config.pushConfig = pushConfig;
    [[ZIM getInstance] callQuit:callID config:config callback:^(NSString * _Nonnull callID, ZIMCallQuitSentInfo * _Nonnull info, ZIMError * _Nonnull errorInfo) {
        callback(callID,info,errorInfo);
        GGLog(@"[GGLog][callback][ZIMCallQuitCallback],callID:%@,code:%lu,message:%@",callID,errorInfo.code,errorInfo.message);
    }];
    GGLog(@"[GGLog][operation][callQuitWithCallID],callID:%@,extendeddata:%@",callID,config.extendedData);
}

- (void)callEnd:(NSString *)callID
         config:(ZIMCallEndConfig *)config
       callback:(ZIMCallEndSentCallback)callback{
    ZIMPushConfig *pushConfig = [[ZIMPushConfig alloc] init];
    pushConfig.title =@"title";
    pushConfig.content = @"cancel";
    pushConfig.payload = @"payload";
    pushConfig.resourcesID = KeyCenter.resourceID;
    config.pushConfig = pushConfig;
    [[ZIM getInstance] callEnd:callID config:config callback:^(NSString * _Nonnull callID, ZIMCallEndedSentInfo * _Nonnull info, ZIMError * _Nonnull errorInfo) {
        GGLog(@"[GGLog][callback][ZIMCallEndCallback],callID:%@,code:%lu,message:%@",callID,errorInfo.code,errorInfo.message);
        callback(callID,info,errorInfo);
    }];

}

- (void)queryCallInvitationListWithConfig:(ZIMCallInvitationQueryConfig *)config callback:(ZIMCallInvitationListQueriedCallback)callback{
    
    [[ZIM getInstance] queryCallInvitationListWithConfig:config callback:^(NSArray<ZIMCallInfo *> * _Nonnull callList, long long nextFlag, ZIMError * _Nonnull errorInfo) {
        for (ZIMCallInfo *callInfo in callList) {
            GGLog(@"callID:%@,caller:%@,,inviter:%@,state:%lu",callInfo.callID,callInfo.caller,callInfo.inviter,callInfo.state);
            for (ZIMCallUserInfo *callUserInfo in callInfo.callUserList) {
                GGLog(@"userID:%@,state:%d,extended data:%@",callUserInfo.userID,callUserInfo.state,callUserInfo.extendedData);
            }
        }
        callback(callList,nextFlag,errorInfo);
    }];
}
// MARK: - ZIMEventHandler

- (void)zim:(ZIM *)zim errorInfo:(ZIMError *)errorInfo{
    GGLog(@"[GGLog][event][errorInfo] code:%d,message:%@",(int)errorInfo.code,errorInfo.message);
}

- (void)zim:(ZIM *)zim connectionStateChanged:(ZIMConnectionState)state event:(ZIMConnectionEvent)event extendedData:(NSDictionary *)extendedData {

    GGLog(@"[GGLog][event][connectionStateChanged] state:%lu,event:%lu",(unsigned long)state,event);
    
    if(state == 0 && event == 4){
        [ZGHUDHelper showMessage:[NSString stringWithFormat:NSLocalizedString(@"Logged out because you logged in from another device.",nil)]];
        NSUserDefaults *userDefault = [NSUserDefaults standardUserDefaults];
        [userDefault setObject:@"" forKey:@"alreadyUserID"];
        [userDefault setObject:@"" forKey:@"alreadyUserName"];
        
        [ConversationListDataSource releaseMySelf];
        LoginViewController *vc = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"LoginViewController"];
        NSMutableArray *controllers = [NSMutableArray arrayWithArray: [UIViewControllerCJHelper findCurrentShowingViewController].navigationController.viewControllers];
        controllers[0] = vc;
        [ZGHUDHelper showNetworkLoading];
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [ZGHUDHelper hideNetworkLoading];
            [[UIViewControllerCJHelper findCurrentShowingViewController].navigationController setViewControllers:controllers];
        });
        [[ZGZIMManager shared] destroyZIM];
        

    }
    // Dispatch event callback
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(connectionStateChanged:event:extendedData:)]) {
                [delegate connectionStateChanged:state event:event extendedData:extendedData];
            }
        }
    });
}

- (void)zim:(ZIM *)zim tokenWillExpire:(unsigned int)second {
    
    UIAlertController *alertView = [UIAlertController alertControllerWithTitle:@"tips" message:@"please input token" preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *action = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        if(self.tokenTextField == nil||[self.tokenTextField.text isEqual:@""] ||self.tokenTextField.text == nil){
            [ZGHUDHelper showMessage:@"The token cannot be null"];
        }else{
            [[ZIM getInstance] renewToken:self.tokenTextField.text callback:^(NSString * _Nonnull token, ZIMError * _Nonnull errorInfo) {
                if(errorInfo.code != 0){
                    [ZGHUDHelper showMessage:[NSString stringWithFormat:@"failed,code:%lu,message:%@",errorInfo.code,errorInfo.message]];
                }
            }];
        }
    }];
        [alertView addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        self.tokenTextField = textField;
    }];
    [alertView addAction:action];
    [[UIApplication sharedApplication].keyWindow.rootViewController presentViewController:alertView animated:YES completion:nil];
    
    GGLog(@"[GGLog][event][tokenWillExpire] second:%u",second);
    // Dispatch event callback
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(tokenWillExpire:)]) {
                [delegate tokenWillExpire:second];
            }
        }
    });
    
    
    
}



- (void)zim:(ZIM *)zim receivePeerMessage:(NSArray<ZIMMessage *> *)messageList fromUserID:(NSString *)fromUserID {
    GGLog(@"[GGLog][event][receivePeerMessage],fromUserID:%@,messageList:%@",fromUserID,[ZIMPluginConverter mZIMMessageList:messageList]);
    // Dispatch message callback
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(receivePeerMessage:fromUserID:)]) {
                [delegate receivePeerMessage:messageList fromUserID:fromUserID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim receiveRoomMessage:(NSArray<ZIMMessage *> *)messageList fromRoomID:(NSString *)fromRoomID {
    GGLog(@"[GGLog][event][receiveRoomMessage],fromRoomID:%@,messageList:%@",fromRoomID,[ZIMPluginConverter mZIMMessageList:messageList]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(receiveRoomMessage:fromRoomID:)]) {
                [delegate receiveRoomMessage:messageList fromRoomID:fromRoomID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim
    receiveGroupMessage:(NSArray<ZIMMessage *> *)messageList
fromGroupID:(NSString *)fromGroupID{
    GGLog(@"[GGLog][event][receiveGroupMessage],fromGroupID:%@,messageList:%@",fromGroupID,[ZIMPluginConverter mZIMMessageList:messageList]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(receiveGroupMessage:fromGroupID:)]) {
                [delegate receiveGroupMessage:messageList fromGroupID:fromGroupID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim broadcastMessageReceived:(ZIMMessage *)message{
    GGLog(@"[GGLog][event][broadcastMessageReceived],message:%@",[ZIMPluginConverter mZIMMessage:message]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:broadcastMessageReceived:)]) {
                [delegate zim:zim broadcastMessageReceived:message];
            }
        }
    });

}

- (void)zim:(ZIM *)zim messageRevokeReceived:(NSArray<ZIMRevokeMessage *> *)messageList{
    GGLog(@"[GGLog][event][messageRevokeReceived,messageList:%@",[ZIMPluginConverter mZIMMessageList:messageList]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(messageRevokeReceived:)]) {
                [delegate messageRevokeReceived:messageList];
            }
        }
    });
}

- (void)zim:(ZIM *)zim messageReceiptChanged:(NSArray<ZIMMessageReceiptInfo *> *)infos{
    NSMutableArray *infosModel = [[NSMutableArray alloc] init];
    for (ZIMMessageReceiptInfo *info in infos) {
        [infosModel safeAddObject:[ZIMPluginConverter mZIMMessageReceiptInfo:info]];
    }
    GGLog(@"[GGLog][event][messageReceiptChanged],infos:%@",infosModel);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if([delegate respondsToSelector:@selector(zim:messageReceiptChanged:)]){
                [delegate zim:zim messageReceiptChanged:infos];
            }
        }
    });
    
}

- (void)zim:(ZIM *)zim
    messageSentStatusChanged:
(NSArray<ZIMMessageSentStatusChangeInfo *> *)messageSentStatusChangeInfoList{
    GGLog(@"[GGLog][event][messageSentStatusChanged],messageSentStatusChangeInfoList:%@",[ZIMPluginConverter mMessageSentStatusChangeInfoList:messageSentStatusChangeInfoList]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if([delegate respondsToSelector:@selector(zim:messageSentStatusChanged:)]){
                [delegate zim:zim messageSentStatusChanged:messageSentStatusChangeInfoList];
            }
        }
    });
    
}

- (void)zim:(ZIM *)zim messageDeleted:(ZIMMessageDeletedInfo *)deletedInfo{
    GGLog(@"[GGLog][event][messageDeleted],isDeleteConversationAllMessage:%@,messageList:%@,conversation id:%@,conversation type:%lu,",deletedInfo.isDeleteConversationAllMessage?@"true":@"false",[ZIMPluginConverter mZIMMessageList:deletedInfo.messageList],deletedInfo.conversationID,(unsigned long)deletedInfo.conversationType);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if([delegate respondsToSelector:@selector(zim:messageDeleted:)]){

                [delegate zim:zim messageDeleted:deletedInfo];
            }
        }
    });
}

- (void)zim:(ZIM *)zim conversationMessageReceiptChanged:(NSArray<ZIMMessageReceiptInfo *> *)infos{
    NSMutableArray *infosModel = [[NSMutableArray alloc] init];
    for (ZIMMessageReceiptInfo *info in infos) {
        [infosModel safeAddObject:[ZIMPluginConverter mZIMMessageReceiptInfo:info]];
    }
    
    GGLog(@"[GGLog][event][conversationMessageReceiptChanged],infos:%@",infosModel);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if([delegate respondsToSelector:@selector(zim:conversationMessageReceiptChanged:)]){
                [delegate zim:zim conversationMessageReceiptChanged:infos];
            }
        }
    });
}

- (void)zim:(ZIM *)zim
conversationChanged:(NSArray<ZIMConversationChangeInfo *> *)conversationChangeInfoList{
    GGLog(@"[GGlog][Event][conversationChanged]:conversationChangeInfoList:%@", [ZIMPluginConverter mConversationChangeInfoList:conversationChangeInfoList]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if([delegate respondsToSelector:@selector(conversationChanged:)]){
            [delegate conversationChanged:conversationChangeInfoList];
            }
        }
    });
    
    
}

- (void)zim:(ZIM *)zim
conversationTotalUnreadMessageCountUpdated:(unsigned int)totalUnreadMessageCount{
    GGLog(@"[GGLog][Event][conversationTotalUnreadMessageCountUpdated] totalUnreadMessageCount:%u",totalUnreadMessageCount);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(conversationTotalUnreadMessageCountUpdated:)]) {
                [delegate conversationTotalUnreadMessageCountUpdated:totalUnreadMessageCount];
            }
        }
    });
}

- (void)zim:(ZIM *)zim roomMemberJoined:(NSArray<ZIMUserInfo *> *)memberList roomID:(NSString *)roomID {
    GGLog(@"[GGlog][Event][roomMemberJoined],memberList:%@,roomID:%@",[ZIMPluginConverter mZIMUserInfoList:memberList],roomID);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(roomMemberJoined:roomID:)]) {
                [delegate roomMemberJoined:memberList roomID:roomID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim roomMemberLeft:(NSArray<ZIMUserInfo *> *)memberList roomID:(NSString *)roomID {
    GGLog(@"[GGlog][Event][roomMemberLeft],memberlist:%@,roomID:%@",[ZIMPluginConverter mZIMUserInfoList:memberList],roomID);
    // Dispatch message callback
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(roomMemberLeft:roomID:)]) {
                [delegate roomMemberLeft:memberList roomID:roomID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim
    roomStateChanged:(ZIMRoomState)state
               event:(ZIMRoomEvent)event
        extendedData:(NSDictionary *)extendedData
     roomID:(NSString *)roomID{
    // Dispatch message callback
    GGLog(@"[GGlog][Event][roomStateChanged],state:%lu,event:%lu,extendedData:%@,roomID:%@",state,event,extendedData,roomID);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(roomStateChanged:event:extendedData:roomID:)]) {
                [delegate roomStateChanged:state event:event extendedData:extendedData roomID:roomID];
            }
            
        }
    });
}

- (void)zim:(ZIM *)zim
    roomAttributesUpdated:(ZIMRoomAttributesUpdateInfo *)updateInfo
     roomID:(NSString *)roomID{
    GGLog(@"[GGlog][Event][roomAttributesUpdated],roomID:%@,updateInfo:%@",roomID,[ZIMPluginConverter mZIMRoomAttributesUpdateInfo:updateInfo]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:roomAttributesUpdated:roomID:)]) {
            [delegate roomAttributesUpdated:updateInfo roomID:roomID];
            }
        }
    });
}



- (void)zim:(ZIM *)zim
    roomAttributesBatchUpdated:(NSArray<ZIMRoomAttributesUpdateInfo *> *)updateInfo
     roomID:(NSString *)roomID {
    GGLog(@"[GGlog][Event][roomAttributesBatchUpdated],roomID:%@,updateInfo:%@",roomID,[ZIMPluginConverter mZIMRoomAttributesUpdateInfoList:updateInfo]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if([delegate respondsToSelector:@selector(roomAttributesBatchUpdated:roomID:)]){
                [delegate roomAttributesBatchUpdated:updateInfo roomID:roomID];
            };
        }
    });
}

//- (void)zim:(ZIM *)zim
//    roomMemberAttributesUpdated:(NSArray<ZIMRoomMemberAttributesUpdateInfo *> *)infos
//                   operatedInfo:(ZIMRoomOperatedInfo *)operatedInfo
//     roomID:(NSString *)roomID{
//    NSMutableArray *infosModel = [[NSMutableArray alloc] init];
//    for (ZIMRoomMemberAttributesUpdateInfo *info in infos) {
//        [infosModel safeAddObject:[ZIMPluginConverter mZIMRoomMemberAttributesUpdateInfo:info]];
//    }
//
//    GGLog(@"[GGLog][Event][roomMemberAttributesUpdated],infos:%@,operatedInfo:%@",infosModel,[ZIMPluginConverter mZIMRoomOperatedInfo:operatedInfo]);
//    dispatch_async(dispatch_get_main_queue(), ^{
//        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
//            if([delegate respondsToSelector:@selector(zim:roomMemberAttributesUpdated:operatedInfo:roomID:)]){
//                [delegate zim:zim roomMemberAttributesUpdated:infos operatedInfo:operatedInfo roomID:roomID];
//            }
//        }
//    });
//}



- (void)zim:(ZIM *)zim
    groupStateChanged:(ZIMGroupState)state
                event:(ZIMGroupEvent)event
         operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
  groupInfo:(ZIMGroupFullInfo *)groupInfo{
    GGLog(@"[GGlog][Event][groupStateChanged],state:%lu,event:%lu,groupInfo:%@,operatedInfo:%@",state,event,[ZIMPluginConverter mZIMGroupFullInfo:groupInfo],[ZIMPluginConverter mZIMGroupOperatedInfo:operatedInfo]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(groupStateChanged:event:operatedInfo:groupInfo:)]) {
                [delegate groupStateChanged:state event:event operatedInfo:operatedInfo groupInfo:groupInfo];
            }
        }
    });
}

- (void)zim:(ZIM *)zim
    groupNameUpdated:(NSString *)groupName
        operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
    groupID:(NSString *)groupID{
    // Dispatch message callback
    GGLog(@"[GGlog][Event][groupNameUpdated],groupID:%@,groupName:%@,operatedInfo:%@",groupID,groupName,[ZIMPluginConverter mZIMGroupOperatedInfo:operatedInfo]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(groupNameUpdated:operatedInfo:groupID:)]) {
                [delegate groupNameUpdated:groupName operatedInfo:operatedInfo groupID:groupID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim
    groupNoticeUpdated:(NSString *)groupNotice
          operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
    groupID:(NSString *)groupID{
    // Dispatch message callback
    GGLog(@"[GGlog][Event][groupNoticeUpdated],groupID:%@,groupNotice:%@,operatedInfo:%@",groupID,groupNotice,[ZIMPluginConverter mZIMGroupOperatedInfo:operatedInfo]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(groupNoticeUpdated:operatedInfo:groupID:)]) {
                [delegate groupNoticeUpdated:groupNotice operatedInfo:operatedInfo groupID:groupID];
            }
            
        }
    });
}

- (void)zim:(ZIM *)zim
    groupAvatarUrlUpdated:(NSString *)groupAvatarUrl
             operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
    groupID:(NSString *)groupID{
    GGLog(@"[GGlog][Event][groupAvatarUrlUpdated],groupAvatarUrl:%@,operatedInfo:%@,groupID:%@",groupAvatarUrl,[ZIMPluginConverter mZIMGroupOperatedInfo:operatedInfo],groupID);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:groupAvatarUrlUpdated:operatedInfo:groupID:)]) {
                [delegate zim:zim groupAvatarUrlUpdated:groupAvatarUrl operatedInfo:operatedInfo groupID:groupID];
            }
            
        }
    });
}


- (void)zim:(ZIM *)zim
    groupAttributesUpdated:(NSArray<ZIMGroupAttributesUpdateInfo *> *)updateInfo
              operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
    groupID:(NSString *)groupID{
    GGLog(@"[GGlog][Event][groupAttributesUpdated],groupID:%@,updateInfo:%@,operatedInfo:%@",groupID,[ZIMPluginConverter mZIMGroupAttributesUpdateInfoList:updateInfo],[ZIMPluginConverter mZIMGroupOperatedInfo:operatedInfo]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(groupAttributesUpdated:operatedInfo:groupID:)]) {
                [delegate groupAttributesUpdated:updateInfo operatedInfo:operatedInfo groupID:groupID];
            }
           
        }
    });
}

- (void)zim:(ZIM *)zim
    groupMemberStateChanged:(ZIMGroupMemberState)state
                      event:(ZIMGroupMemberEvent)event
                   userList:(NSArray<ZIMGroupMemberInfo *> *)userList
               operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
    groupID:(NSString *)groupID{
    GGLog(@"[GGlog][Event][groupMemberStateChanged],groupID:%@,event:%lu,state:%lu,userList:%@,operatedInfo:%@",groupID,(unsigned long)event,(unsigned long)state,[ZIMPluginConverter mZIMGroupMemberInfoList:userList],[ZIMPluginConverter mZIMGroupOperatedInfo:operatedInfo]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(groupMemberStateChanged:event:userList:operatedInfo:groupID:)]) {
                [delegate groupMemberStateChanged:state event:event userList:userList operatedInfo:operatedInfo groupID:groupID];
            }
            
        }
    });
}

- (void)zim:(ZIM *)zim
    groupMemberInfoUpdated:(NSArray<ZIMGroupMemberInfo *> *)userInfo
              operatedInfo:(ZIMGroupOperatedInfo *)operatedInfo
    groupID:(NSString *)groupID{
    GGLog(@"[GGlog][Event][groupMemberInfoUpdated],groupID:%@,userInfo:%@,operatedInfo:%@",groupID,[ZIMPluginConverter mZIMGroupMemberInfoList:userInfo],[ZIMPluginConverter mZIMGroupOperatedInfo:operatedInfo]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(groupMemberInfoUpdated:operatedInfo:groupID:)]) {
                [delegate groupMemberInfoUpdated:userInfo operatedInfo:operatedInfo groupID:groupID];
            }
        }
    });
}


- (void)zim:(ZIM *)zim
    callInvitationReceived:(ZIMCallInvitationReceivedInfo *)info
     callID:(NSString *)callID{
    [CallInviteView show:Receiver userID:info.inviter callID:callID mode:info.mode];
    self.targetCallID = callID;
    GGLog(@"[GGlog][Event][callInvitationReceived],callID:%@,inviter:%@,timeout:%d,extendedData:%@",callID,info.inviter,info.timeout,info.extendedData);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:callInvitationReceived:callID:)]) {
                [delegate zim:zim callInvitationReceived:info callID:callID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim
    callInvitationCancelled:(ZIMCallInvitationCancelledInfo *)info
     callID:(NSString *)callID{
    [CallInviteView CallerCancel:callID];
    GGLog(@"[GGlog][Event][callInvitationCancelled],callID:%@,inviter:%@,,extendedData:%@,mode:%ld",callID,info.inviter,info.extendedData,(long)info.mode);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:callInvitationCancelled:callID:)]) {
                [delegate zim:zim callInvitationCancelled:info callID:callID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim
    callInvitationAccepted:(ZIMCallInvitationAcceptedInfo *)info
     callID:(NSString *)callID{
    [CallInviteView receiverAccept:callID];
    GGLog(@"[GGlog][Event][callInvitationAccepted],callID:%@,invitee:%@,,extendedData:%@",callID,info.invitee,info.extendedData);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:callInvitationAccepted:callID:)]) {
                [delegate zim:zim callInvitationAccepted:info callID:callID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim
    callInvitationRejected:(ZIMCallInvitationRejectedInfo *)info
     callID:(NSString *)callID{
    [CallInviteView receiverRefused:callID];
    GGLog(@"[GGlog][Event][callInvitationRejected],callID:%@,invitee:%@,,extendedData:%@",callID,info.invitee,info.extendedData);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:callInvitationRejected:callID:)]) {
                [delegate zim:zim callInvitationRejected:info callID:callID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim callInvitationTimeout:(NSString *)callID{
    [CallInviteView timeout:callID];
    GGLog(@"[GGlog][Event][callInvitationTimeout],callID:%@",callID);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:callInvitationTimeout:)]) {
                [delegate zim:zim callInvitationTimeout:callID];
            }
        }
    });
}


- (void)zim:(ZIM *)zim
    callInviteesAnsweredTimeout:(NSArray<NSString *> *)invitees
     callID:(NSString *)callID{
    [CallInviteView timeout:callID];
    GGLog(@"[GGlog][Event][callInviteesAnsweredTimeout],callID:%@,invitees:%@",callID,invitees);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:callInviteesAnsweredTimeout:callID:)]) {
                [delegate zim:zim callInviteesAnsweredTimeout:invitees callID:callID];
            }
        }
    });
}


- (void)zim:(ZIM *)zim callUserStateChanged:(ZIMCallUserStateChangeInfo *)info callID:(nonnull NSString *)callID{
    GGLog(@"[GGlog][Event][callUserStateChanged],callID:%@,callUserList:%@",callID,[ZIMPluginConverter mZIMCallUserInfoList:info.callUserList]);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:callUserStateChanged:callID:)]) {
                [delegate zim:zim callUserStateChanged:info callID:callID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim callInvitationEnded:(ZIMCallInvitationEndedInfo *)info callID:(NSString *)callID{
    GGLog(@"[GGlog][Event][callInvitationEnded],callID:%@,caller:%@,operatedUserID:%@,extendedData:%@,end time:%lld,mode:%ld",callID,info.caller,info.operatedUserID,info.extendedData,info.endTime,(long)info.mode);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:callInvitationEnded:callID:)]) {
                [delegate zim:zim callInvitationEnded:info callID:callID];
            }
        }
    });
}

- (void)zim:(ZIM *)zim userInfoUpdated:(ZIMUserFullInfo *)info{
    GGLog(@"[GGLog][Event][userInfoUpdated],userID:%@,userName:%@,userAvatarUrl:%@,extendedData:%@",info.baseInfo.userID,info.baseInfo.userName,info.userAvatarUrl,info.extendedData);
    dispatch_async(dispatch_get_main_queue(), ^{
        for (id<ZIMEventDelegate> delegate in self.zimEventDelegates) {
            if ([delegate respondsToSelector:@selector(zim:userInfoUpdated:)]) {
                [delegate zim:zim userInfoUpdated:info];
            }
        }
    });
}
@end
